from datetime import date
import json

import ddt

import falcon
from falcon.errors import HTTPInvalidParam
import falcon.testing as testing


class Resource(testing.SimpleTestResource):

    @falcon.before(testing.capture_responder_args)
    @falcon.before(testing.set_resp_defaults)
    def on_put(self, req, resp, **kwargs):
        pass

    @falcon.before(testing.capture_responder_args)
    @falcon.before(testing.set_resp_defaults)
    def on_patch(self, req, resp, **kwargs):
        pass

    @falcon.before(testing.capture_responder_args)
    @falcon.before(testing.set_resp_defaults)
    def on_delete(self, req, resp, **kwargs):
        pass

    @falcon.before(testing.capture_responder_args)
    @falcon.before(testing.set_resp_defaults)
    def on_head(self, req, resp, **kwargs):
        pass

    @falcon.before(testing.capture_responder_args)
    @falcon.before(testing.set_resp_defaults)
    def on_options(self, req, resp, **kwargs):
        pass


@ddt.ddt
class _TestQueryParams(testing.TestBase):

    def before(self):
        self.resource = Resource()
        self.api.add_route('/', self.resource)

    def test_none(self):
        query_string = ''
        self.simulate_request('/', query_string=query_string)

        req = self.resource.captured_req
        store = {}
        self.assertIs(req.get_param('marker'), None)
        self.assertIs(req.get_param('limit', store), None)
        self.assertNotIn('limit', store)
        self.assertIs(req.get_param_as_int('limit'), None)
        self.assertIs(req.get_param_as_bool('limit'), None)
        self.assertIs(req.get_param_as_list('limit'), None)

    def test_blank(self):
        query_string = 'marker='
        self.simulate_request('/', query_string=query_string)

        req = self.resource.captured_req
        self.assertIs(req.get_param('marker'), None)

        store = {}
        self.assertIs(req.get_param('marker', store=store), None)
        self.assertNotIn('marker', store)

    def test_simple(self):
        query_string = 'marker=deadbeef&limit=25'
        self.simulate_request('/', query_string=query_string)

        req = self.resource.captured_req
        store = {}
        self.assertEqual(req.get_param('marker', store=store) or 'nada',
                         'deadbeef')
        self.assertEqual(req.get_param('limit', store=store) or '0', '25')

        self.assertEqual(store['marker'], 'deadbeef')
        self.assertEqual(store['limit'], '25')

    def test_percent_encoded(self):
        query_string = 'id=23,42&q=%e8%b1%86+%e7%93%a3'
        self.simulate_request('/', query_string=query_string)

        req = self.resource.captured_req

        # NOTE(kgriffs): For lists, get_param will return one of the
        # elements, but which one it will choose is undefined.
        self.assertIn(req.get_param('id'), [u'23', u'42'])

        self.assertEqual(req.get_param_as_list('id', int), [23, 42])
        self.assertEqual(req.get_param('q'), u'\u8c46 \u74e3')

    def test_option_auto_parse_qs_csv_simple_false(self):
        self.api.req_options.auto_parse_qs_csv = False

        query_string = 'id=23,42,,&id=2'
        self.simulate_request('/', query_string=query_string)

        req = self.resource.captured_req

        self.assertEqual(req.params['id'], [u'23,42,,', u'2'])
        self.assertIn(req.get_param('id'), [u'23,42,,', u'2'])
        self.assertEqual(req.get_param_as_list('id'), [u'23,42,,', u'2'])

    def test_option_auto_parse_qs_csv_simple_true(self):
        self.api.req_options.auto_parse_qs_csv = True

        query_string = 'id=23,42,,&id=2'
        self.simulate_request('/', query_string=query_string)

        req = self.resource.captured_req

        self.assertEqual(req.params['id'], [u'23', u'42', u'2'])
        self.assertIn(req.get_param('id'), [u'23', u'42', u'2'])
        self.assertEqual(req.get_param_as_list('id', int), [23, 42, 2])

    def test_option_auto_parse_qs_csv_complex_false(self):
        self.api.req_options.auto_parse_qs_csv = False

        encoded_json = '%7B%22msg%22:%22Testing%201,2,3...%22,%22code%22:857%7D'
        decoded_json = '{"msg":"Testing 1,2,3...","code":857}'

        query_string = ('colors=red,green,blue&limit=1'
                        '&list-ish1=f,,x&list-ish2=,0&list-ish3=a,,,b'
                        '&empty1=&empty2=,&empty3=,,'
                        '&thing=' + encoded_json)

        self.simulate_request('/', query_string=query_string)

        req = self.resource.captured_req

        self.assertIn(req.get_param('colors'), 'red,green,blue')
        self.assertEqual(req.get_param_as_list('colors'), [u'red,green,blue'])

        self.assertEqual(req.get_param_as_list('limit'), ['1'])

        self.assertEqual(req.get_param_as_list('empty1'), None)
        self.assertEqual(req.get_param_as_list('empty2'), [u','])
        self.assertEqual(req.get_param_as_list('empty3'), [u',,'])

        self.assertEqual(req.get_param_as_list('list-ish1'), [u'f,,x'])
        self.assertEqual(req.get_param_as_list('list-ish2'), [u',0'])
        self.assertEqual(req.get_param_as_list('list-ish3'), [u'a,,,b'])

        self.assertEqual(req.get_param('thing'), decoded_json)

    def test_bad_percentage(self):
        query_string = 'x=%%20%+%&y=peregrine&z=%a%z%zz%1%20e'
        self.simulate_request('/', query_string=query_string)
        self.assertEqual(self.srmock.status, falcon.HTTP_200)

        req = self.resource.captured_req
        self.assertEqual(req.get_param('x'), '% % %')
        self.assertEqual(req.get_param('y'), 'peregrine')
        self.assertEqual(req.get_param('z'), '%a%z%zz%1 e')

    def test_allowed_names(self):
        query_string = ('p=0&p1=23&2p=foo&some-thing=that&blank=&'
                        'some_thing=x&-bogus=foo&more.things=blah&'
                        '_thing=42&_charset_=utf-8')
        self.simulate_request('/', query_string=query_string)

        req = self.resource.captured_req
        self.assertEqual(req.get_param('p'), '0')
        self.assertEqual(req.get_param('p1'), '23')
        self.assertEqual(req.get_param('2p'), 'foo')
        self.assertEqual(req.get_param('some-thing'), 'that')
        self.assertIs(req.get_param('blank'), None)
        self.assertEqual(req.get_param('some_thing'), 'x')
        self.assertEqual(req.get_param('-bogus'), 'foo')
        self.assertEqual(req.get_param('more.things'), 'blah')
        self.assertEqual(req.get_param('_thing'), '42')
        self.assertEqual(req.get_param('_charset_'), 'utf-8')

    @ddt.data('get_param', 'get_param_as_int', 'get_param_as_bool',
              'get_param_as_list')
    def test_required(self, method_name):
        query_string = ''
        self.simulate_request('/', query_string=query_string)

        req = self.resource.captured_req

        try:
            getattr(req, method_name)('marker', required=True)
            self.fail('falcon.HTTPMissingParam not raised')
        except falcon.HTTPMissingParam as ex:
            self.assertIsInstance(ex, falcon.HTTPBadRequest)
            self.assertEqual(ex.title, 'Missing parameter')
            expected_desc = 'The "marker" parameter is required.'
            self.assertEqual(ex.description, expected_desc)

    def test_int(self):
        query_string = 'marker=deadbeef&limit=25'
        self.simulate_request('/', query_string=query_string)

        req = self.resource.captured_req

        try:
            req.get_param_as_int('marker')
        except Exception as ex:
            self.assertIsInstance(ex, falcon.HTTPBadRequest)
            self.assertIsInstance(ex, falcon.HTTPInvalidParam)
            self.assertEqual(ex.title, 'Invalid parameter')
            expected_desc = ('The "marker" parameter is invalid. '
                             'The value must be an integer.')
            self.assertEqual(ex.description, expected_desc)

        self.assertEqual(req.get_param_as_int('limit'), 25)

        store = {}
        self.assertEqual(req.get_param_as_int('limit', store=store), 25)
        self.assertEqual(store['limit'], 25)

        self.assertEqual(
            req.get_param_as_int('limit', min=1, max=50), 25)

        self.assertRaises(
            falcon.HTTPBadRequest,
            req.get_param_as_int, 'limit', min=0, max=10)

        self.assertRaises(
            falcon.HTTPBadRequest,
            req.get_param_as_int, 'limit', min=0, max=24)

        self.assertRaises(
            falcon.HTTPBadRequest,
            req.get_param_as_int, 'limit', min=30, max=24)

        self.assertRaises(
            falcon.HTTPBadRequest,
            req.get_param_as_int, 'limit', min=30, max=50)

        self.assertEqual(
            req.get_param_as_int('limit', min=1), 25)

        self.assertEqual(
            req.get_param_as_int('limit', max=50), 25)

        self.assertEqual(
            req.get_param_as_int('limit', max=25), 25)

        self.assertEqual(
            req.get_param_as_int('limit', max=26), 25)

        self.assertEqual(
            req.get_param_as_int('limit', min=25), 25)

        self.assertEqual(
            req.get_param_as_int('limit', min=24), 25)

        self.assertEqual(
            req.get_param_as_int('limit', min=-24), 25)

    def test_int_neg(self):
        query_string = 'marker=deadbeef&pos=-7'
        self.simulate_request('/', query_string=query_string)

        req = self.resource.captured_req
        self.assertEqual(req.get_param_as_int('pos'), -7)

        self.assertEqual(
            req.get_param_as_int('pos', min=-10, max=10), -7)

        self.assertEqual(
            req.get_param_as_int('pos', max=10), -7)

        self.assertRaises(
            falcon.HTTPBadRequest,
            req.get_param_as_int, 'pos', min=-6, max=0)

        self.assertRaises(
            falcon.HTTPBadRequest,
            req.get_param_as_int, 'pos', min=-6)

        self.assertRaises(
            falcon.HTTPBadRequest,
            req.get_param_as_int, 'pos', min=0, max=10)

        self.assertRaises(
            falcon.HTTPBadRequest,
            req.get_param_as_int, 'pos', min=0, max=10)

    def test_boolean(self):
        query_string = ('echo=true&doit=false&bogus=bar&bogus2=foo&'
                        't1=True&f1=False&t2=yes&f2=no&blank&one=1&zero=0&'
                        'checkbox1=on&checkbox2=off')
        self.simulate_request('/', query_string=query_string)

        req = self.resource.captured_req
        self.assertRaises(falcon.HTTPBadRequest, req.get_param_as_bool,
                          'bogus')

        try:
            req.get_param_as_bool('bogus2')
        except Exception as ex:
            self.assertIsInstance(ex, falcon.HTTPInvalidParam)
            self.assertEqual(ex.title, 'Invalid parameter')
            expected_desc = ('The "bogus2" parameter is invalid. '
                             'The value of the parameter must be "true" '
                             'or "false".')
            self.assertEqual(ex.description, expected_desc)

        self.assertEqual(req.get_param_as_bool('echo'), True)
        self.assertEqual(req.get_param_as_bool('doit'), False)

        self.assertEqual(req.get_param_as_bool('t1'), True)
        self.assertEqual(req.get_param_as_bool('t2'), True)
        self.assertEqual(req.get_param_as_bool('f1'), False)
        self.assertEqual(req.get_param_as_bool('f2'), False)
        self.assertEqual(req.get_param_as_bool('one'), True)
        self.assertEqual(req.get_param_as_bool('zero'), False)
        self.assertEqual(req.get_param('blank'), None)

        self.assertEqual(req.get_param_as_bool('checkbox1'), True)
        self.assertEqual(req.get_param_as_bool('checkbox2'), False)

        store = {}
        self.assertEqual(req.get_param_as_bool('echo', store=store), True)
        self.assertEqual(store['echo'], True)

    def test_boolean_blank(self):
        self.api.req_options.keep_blank_qs_values = True
        self.simulate_request(
            '/',
            query_string='blank&blank2=',
        )

        req = self.resource.captured_req
        self.assertEqual(req.get_param('blank'), '')
        self.assertEqual(req.get_param('blank2'), '')
        self.assertRaises(falcon.HTTPInvalidParam, req.get_param_as_bool,
                          'blank')
        self.assertRaises(falcon.HTTPInvalidParam, req.get_param_as_bool,
                          'blank2')
        self.assertEqual(req.get_param_as_bool('blank', blank_as_true=True),
                         True)
        self.assertEqual(req.get_param_as_bool('blank3', blank_as_true=True),
                         None)

    def test_list_type(self):
        query_string = ('colors=red,green,blue&limit=1'
                        '&list-ish1=f,,x&list-ish2=,0&list-ish3=a,,,b'
                        '&empty1=&empty2=,&empty3=,,'
                        '&thing_one=1,,3'
                        '&thing_two=1&thing_two=&thing_two=3')
        self.simulate_request('/', query_string=query_string)

        req = self.resource.captured_req

        # NOTE(kgriffs): For lists, get_param will return one of the
        # elements, but which one it will choose is undefined.
        self.assertIn(req.get_param('colors'), ('red', 'green', 'blue'))

        self.assertEqual(req.get_param_as_list('colors'),
                         ['red', 'green', 'blue'])
        self.assertEqual(req.get_param_as_list('limit'), ['1'])
        self.assertIs(req.get_param_as_list('marker'), None)

        self.assertEqual(req.get_param_as_list('empty1'), None)
        self.assertEqual(req.get_param_as_list('empty2'), [])
        self.assertEqual(req.get_param_as_list('empty3'), [])

        self.assertEqual(req.get_param_as_list('list-ish1'),
                         ['f', 'x'])

        # Ensure that '0' doesn't get translated to None
        self.assertEqual(req.get_param_as_list('list-ish2'),
                         ['0'])

        # Ensure that '0' doesn't get translated to None
        self.assertEqual(req.get_param_as_list('list-ish3'),
                         ['a', 'b'])

        # Ensure consistency between list conventions
        self.assertEqual(req.get_param_as_list('thing_one'),
                         ['1', '3'])
        self.assertEqual(req.get_param_as_list('thing_one'),
                         req.get_param_as_list('thing_two'))

        store = {}
        self.assertEqual(req.get_param_as_list('limit', store=store), ['1'])
        self.assertEqual(store['limit'], ['1'])

    def test_list_type_blank(self):
        query_string = ('colors=red,green,blue&limit=1'
                        '&list-ish1=f,,x&list-ish2=,0&list-ish3=a,,,b'
                        '&empty1=&empty2=,&empty3=,,'
                        '&thing_one=1,,3'
                        '&thing_two=1&thing_two=&thing_two=3'
                        '&empty4=&empty4&empty4='
                        '&empty5&empty5&empty5')
        self.api.req_options.keep_blank_qs_values = True
        self.simulate_request(
            '/',
            query_string=query_string
        )

        req = self.resource.captured_req

        # NOTE(kgriffs): For lists, get_param will return one of the
        # elements, but which one it will choose is undefined.
        self.assertIn(req.get_param('colors'), ('red', 'green', 'blue'))

        self.assertEqual(req.get_param_as_list('colors'),
                         ['red', 'green', 'blue'])
        self.assertEqual(req.get_param_as_list('limit'), ['1'])
        self.assertIs(req.get_param_as_list('marker'), None)

        self.assertEqual(req.get_param_as_list('empty1'), [''])
        self.assertEqual(req.get_param_as_list('empty2'), ['', ''])
        self.assertEqual(req.get_param_as_list('empty3'), ['', '', ''])

        self.assertEqual(req.get_param_as_list('list-ish1'),
                         ['f', '', 'x'])

        # Ensure that '0' doesn't get translated to None
        self.assertEqual(req.get_param_as_list('list-ish2'),
                         ['', '0'])

        # Ensure that '0' doesn't get translated to None
        self.assertEqual(req.get_param_as_list('list-ish3'),
                         ['a', '', '', 'b'])

        # Ensure consistency between list conventions
        self.assertEqual(req.get_param_as_list('thing_one'),
                         ['1', '', '3'])
        self.assertEqual(req.get_param_as_list('thing_one'),
                         req.get_param_as_list('thing_two'))

        store = {}
        self.assertEqual(req.get_param_as_list('limit', store=store), ['1'])
        self.assertEqual(store['limit'], ['1'])

        # Test empty elements
        self.assertEqual(req.get_param_as_list('empty4'), ['', '', ''])
        self.assertEqual(req.get_param_as_list('empty5'), ['', '', ''])
        self.assertEqual(req.get_param_as_list('empty4'),
                         req.get_param_as_list('empty5'))

    def test_list_transformer(self):
        query_string = 'coord=1.4,13,15.1&limit=100&things=4,,1'
        self.simulate_request('/', query_string=query_string)

        req = self.resource.captured_req

        # NOTE(kgriffs): For lists, get_param will return one of the
        # elements, but which one it will choose is undefined.
        self.assertIn(req.get_param('coord'), ('1.4', '13', '15.1'))

        expected = [1.4, 13.0, 15.1]
        actual = req.get_param_as_list('coord', transform=float)
        self.assertEqual(actual, expected)

        expected = ['4', '1']
        actual = req.get_param_as_list('things', transform=str)
        self.assertEqual(actual, expected)

        expected = [4, 1]
        actual = req.get_param_as_list('things', transform=int)
        self.assertEqual(actual, expected)

        try:
            req.get_param_as_list('coord', transform=int)
        except Exception as ex:
            self.assertIsInstance(ex, falcon.HTTPInvalidParam)
            self.assertEqual(ex.title, 'Invalid parameter')
            expected_desc = ('The "coord" parameter is invalid. '
                             'The value is not formatted correctly.')
            self.assertEqual(ex.description, expected_desc)

    def test_param_property(self):
        query_string = 'ant=4&bee=3&cat=2&dog=1'
        self.simulate_request('/', query_string=query_string)

        req = self.resource.captured_req
        self.assertEqual(
            sorted(req.params.items()),
            [('ant', '4'), ('bee', '3'), ('cat', '2'), ('dog', '1')])

    def test_multiple_form_keys(self):
        query_string = 'ant=1&ant=2&bee=3&cat=6&cat=5&cat=4'
        self.simulate_request('/', query_string=query_string)

        req = self.resource.captured_req
        # By definition, we cannot guarantee which of the multiple keys will
        # be returned by .get_param().
        self.assertIn(req.get_param('ant'), ('1', '2'))
        # There is only one 'bee' key so it remains a scalar.
        self.assertEqual(req.get_param('bee'), '3')
        # There are three 'cat' keys; order is preserved.
        self.assertIn(req.get_param('cat'), ('6', '5', '4'))

    def test_multiple_keys_as_bool(self):
        query_string = 'ant=true&ant=yes&ant=True'
        self.simulate_request('/', query_string=query_string)
        req = self.resource.captured_req
        self.assertEqual(req.get_param_as_bool('ant'), True)

    def test_multiple_keys_as_int(self):
        query_string = 'ant=1&ant=2&ant=3'
        self.simulate_request('/', query_string=query_string)
        req = self.resource.captured_req
        self.assertIn(req.get_param_as_int('ant'), (1, 2, 3))

    def test_multiple_form_keys_as_list(self):
        query_string = 'ant=1&ant=2&bee=3&cat=6&cat=5&cat=4'
        self.simulate_request('/', query_string=query_string)

        req = self.resource.captured_req
        # There are two 'ant' keys.
        self.assertEqual(req.get_param_as_list('ant'), ['1', '2'])
        # There is only one 'bee' key..
        self.assertEqual(req.get_param_as_list('bee'), ['3'])
        # There are three 'cat' keys; order is preserved.
        self.assertEqual(req.get_param_as_list('cat'), ['6', '5', '4'])

    def test_get_date_valid(self):
        date_value = '2015-04-20'
        query_string = 'thedate={0}'.format(date_value)
        self.simulate_request('/', query_string=query_string)
        req = self.resource.captured_req
        self.assertEqual(req.get_param_as_date('thedate'),
                         date(2015, 4, 20))

    def test_get_date_missing_param(self):
        query_string = 'notthedate=2015-04-20'
        self.simulate_request('/', query_string=query_string)
        req = self.resource.captured_req
        self.assertEqual(req.get_param_as_date('thedate'),
                         None)

    def test_get_date_valid_with_format(self):
        date_value = '20150420'
        query_string = 'thedate={0}'.format(date_value)
        format_string = '%Y%m%d'
        self.simulate_request('/', query_string=query_string)
        req = self.resource.captured_req
        self.assertEqual(req.get_param_as_date('thedate',
                         format_string=format_string),
                         date(2015, 4, 20))

    def test_get_date_store(self):
        date_value = '2015-04-20'
        query_string = 'thedate={0}'.format(date_value)
        self.simulate_request('/', query_string=query_string)
        req = self.resource.captured_req
        store = {}
        req.get_param_as_date('thedate', store=store)
        self.assertNotEqual(len(store), 0)

    def test_get_date_invalid(self):
        date_value = 'notarealvalue'
        query_string = 'thedate={0}'.format(date_value)
        format_string = '%Y%m%d'
        self.simulate_request('/', query_string=query_string)
        req = self.resource.captured_req
        self.assertRaises(HTTPInvalidParam, req.get_param_as_date,
                          'thedate', format_string=format_string)

    def test_get_dict_valid(self):
        payload_dict = {'foo': 'bar'}
        query_string = 'payload={0}'.format(json.dumps(payload_dict))
        self.simulate_request('/', query_string=query_string)
        req = self.resource.captured_req
        self.assertEqual(req.get_param_as_dict('payload'),
                         payload_dict)

    def test_get_dict_missing_param(self):
        payload_dict = {'foo': 'bar'}
        query_string = 'notthepayload={0}'.format(json.dumps(payload_dict))
        self.simulate_request('/', query_string=query_string)
        req = self.resource.captured_req
        self.assertEqual(req.get_param_as_dict('payload'),
                         None)

    def test_get_dict_store(self):
        payload_dict = {'foo': 'bar'}
        query_string = 'payload={0}'.format(json.dumps(payload_dict))
        self.simulate_request('/', query_string=query_string)
        req = self.resource.captured_req
        store = {}
        req.get_param_as_dict('payload', store=store)
        self.assertNotEqual(len(store), 0)

    def test_get_dict_invalid(self):
        payload_dict = 'foobar'
        query_string = 'payload={0}'.format(payload_dict)
        self.simulate_request('/', query_string=query_string)
        req = self.resource.captured_req
        self.assertRaises(HTTPInvalidParam, req.get_param_as_dict,
                          'payload')


@ddt.ddt
class PostQueryParams(_TestQueryParams):
    def before(self):
        super(PostQueryParams, self).before()
        self.api.req_options.auto_parse_form_urlencoded = True

    def simulate_request(self, path, query_string, **kwargs):
        headers = kwargs.setdefault('headers', {})
        headers['Content-Type'] = 'application/x-www-form-urlencoded'

        if 'method' not in kwargs:
            kwargs['method'] = 'POST'

        super(PostQueryParams, self).simulate_request(
            path,
            body=query_string,
            **kwargs
        )

    @ddt.data('POST', 'PUT', 'PATCH', 'DELETE', 'OPTIONS')
    def test_http_methods_body_expected(self, http_method):
        query_string = 'marker=deadbeef&limit=25'
        self.simulate_request('/', query_string=query_string,
                              method=http_method)

        req = self.resource.captured_req
        self.assertEqual(req.get_param('marker'), 'deadbeef')
        self.assertEqual(req.get_param('limit'), '25')

    @ddt.data('GET', 'HEAD')
    def test_http_methods_body_not_expected(self, http_method):
        query_string = 'marker=deadbeef&limit=25'
        self.simulate_request('/', query_string=query_string,
                              method=http_method)

        req = self.resource.captured_req
        self.assertEqual(req.get_param('marker'), None)
        self.assertEqual(req.get_param('limit'), None)

    def test_non_ascii(self):
        value = u'\u8c46\u74e3'
        query_string = b'q=' + value.encode('utf-8')
        self.simulate_request('/', query_string=query_string)

        req = self.resource.captured_req
        self.assertIs(req.get_param('q'), None)

    def test_empty_body(self):
        self.simulate_request('/', query_string=None)

        req = self.resource.captured_req
        self.assertIs(req.get_param('q'), None)

    def test_empty_body_no_content_length(self):
        self.simulate_request('/', query_string=None)

        req = self.resource.captured_req
        self.assertIs(req.get_param('q'), None)

    def test_explicitly_disable_auto_parse(self):
        self.api.req_options.auto_parse_form_urlencoded = False
        self.simulate_request('/', query_string='q=42')

        req = self.resource.captured_req
        self.assertIs(req.get_param('q'), None)


class GetQueryParams(_TestQueryParams):
    def simulate_request(self, path, query_string, **kwargs):
        super(GetQueryParams, self).simulate_request(
            path, query_string=query_string, **kwargs)


class PostQueryParamsDefaultBehavior(testing.TestBase):
    def test_dont_auto_parse_by_default(self):
        self.resource = testing.SimpleTestResource()
        self.api.add_route('/', self.resource)

        headers = {'Content-Type': 'application/x-www-form-urlencoded'}
        self.simulate_request('/', body='q=42', headers=headers)

        req = self.resource.captured_req
        self.assertIs(req.get_param('q'), None)



# Copyright 2013 by Rackspace Hosting, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Routing utilities."""

import re

import six

from falcon import HTTP_METHODS, responders


# NOTE(kgriffs): Published method; take care to avoid breaking changes.
def compile_uri_template(template):
    """Compile the given URI template string into a pattern matcher.

    This function can be used to construct custom routing engines that
    iterate through a list of possible routes, attempting to match
    an incoming request against each route's compiled regular expression.

    Each field is converted to a named group, so that when a match
    is found, the fields can be easily extracted using
    :py:meth:`re.MatchObject.groupdict`.

    This function does not support the more flexible templating
    syntax used in the default router. Only simple paths with bracketed
    field expressions are recognized. For example::

        /
        /books
        /books/{isbn}
        /books/{isbn}/characters
        /books/{isbn}/characters/{name}

    Also, note that if the template contains a trailing slash character,
    it will be stripped in order to normalize the routing logic.

    Args:
        template(str): The template to compile. Note that field names are
            restricted to ASCII a-z, A-Z, and the underscore character.

    Returns:
        tuple: (template_field_names, template_regex)
    """

    if not isinstance(template, six.string_types):
        raise TypeError('uri_template is not a string')

    if not template.startswith('/'):
        raise ValueError("uri_template must start with '/'")

    if '//' in template:
        raise ValueError("uri_template may not contain '//'")

    if template != '/' and template.endswith('/'):
        template = template[:-1]

    # template names should be able to start with A-Za-z
    # but also contain 0-9_ in the remaining portion
    expression_pattern = r'{([a-zA-Z]\w*)}'

    # Get a list of field names
    fields = set(re.findall(expression_pattern, template))

    # Convert Level 1 var patterns to equivalent named regex groups
    escaped = re.sub(r'[\.\(\)\[\]\?\*\+\^\|]', r'\\\g<0>', template)
    pattern = re.sub(expression_pattern, r'(?P<\1>[^/]+)', escaped)
    pattern = r'\A' + pattern + r'\Z'

    return fields, re.compile(pattern, re.IGNORECASE)


def create_http_method_map(resource):
    """Maps HTTP methods (e.g., 'GET', 'POST') to methods of a resource object.

    Args:
        resource: An object with *responder* methods, following the naming
            convention *on_\**, that correspond to each method the resource
            supports. For example, if a resource supports GET and POST, it
            should define ``on_get(self, req, resp)`` and
            ``on_post(self, req, resp)``.

    Returns:
        dict: A mapping of HTTP methods to responders.

    """

    method_map = {}

    for method in HTTP_METHODS:
        try:
            responder = getattr(resource, 'on_' + method.lower())
        except AttributeError:
            # resource does not implement this method
            pass
        else:
            # Usually expect a method, but any callable will do
            if callable(responder):
                method_map[method] = responder

    # Attach a resource for unsupported HTTP methods
    allowed_methods = sorted(list(method_map.keys()))

    if 'OPTIONS' not in method_map:
        # OPTIONS itself is intentionally excluded from the Allow header
        opt_responder = responders.create_default_options(allowed_methods)
        method_map['OPTIONS'] = opt_responder
        allowed_methods.append('OPTIONS')

    na_responder = responders.create_method_not_allowed(allowed_methods)

    for method in HTTP_METHODS:
        if method not in allowed_methods:
            method_map[method] = na_responder

    return method_map



from falcon import Response
import falcon.testing as testing


class TestRequestContext(testing.TestBase):

    def test_default_response_context(self):
        resp = Response()
        self.assertIsInstance(resp.context, dict)

    def test_custom_response_context(self):

        class MyCustomContextType(object):
            pass

        class MyCustomResponse(Response):
            context_type = MyCustomContextType

        resp = MyCustomResponse()
        self.assertIsInstance(resp.context, MyCustomContextType)

    def test_custom_response_context_failure(self):

        class MyCustomResponse(Response):
            context_type = False

        self.assertRaises(TypeError, MyCustomResponse)

    def test_custom_response_context_factory(self):

        def create_context(resp):
            return {'resp': resp}

        class MyCustomResponse(Response):
            context_type = create_context

        resp = MyCustomResponse()
        self.assertIsInstance(resp.context, dict)
        self.assertIs(resp.context['resp'], resp)



# Copyright 2013 by Rackspace Hosting, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Hook decorators."""

from functools import wraps

import six

from falcon import HTTP_METHODS
from falcon.util.misc import get_argnames


def before(action):
    """Decorator to execute the given action function *before* the responder.

    Args:
        action (callable): A function of the form
            ``func(req, resp, resource, params)``, where `resource` is a
            reference to the resource class instance associated with the
            request, and `params` is a dict of URI Template field names,
            if any, that will be passed into the resource responder as
            kwargs.

            Note:
                Hooks may inject extra params as needed. For example::

                    def do_something(req, resp, resource, params):
                        try:
                            params['id'] = int(params['id'])
                        except ValueError:
                            raise falcon.HTTPBadRequest('Invalid ID',
                                                        'ID was not valid.')

                        params['answer'] = 42

    """

    def _before(responder_or_resource):
        if isinstance(responder_or_resource, six.class_types):
            resource = responder_or_resource

            for method in HTTP_METHODS:
                responder_name = 'on_' + method.lower()

                try:
                    responder = getattr(resource, responder_name)
                except AttributeError:
                    # resource does not implement this method
                    pass
                else:
                    # Usually expect a method, but any callable will do
                    if callable(responder):
                        # This pattern is necessary to capture the current
                        # value of responder in the do_before_all closure;
                        # otherwise, they will capture the same responder
                        # variable that is shared between iterations of the
                        # for loop, above.
                        def let(responder=responder):
                            do_before_all = _wrap_with_before(action, responder)

                            setattr(resource, responder_name, do_before_all)

                        let()

            return resource

        else:
            responder = responder_or_resource
            do_before_one = _wrap_with_before(action, responder)

            return do_before_one

    return _before


def after(action):
    """Decorator to execute the given action function *after* the responder.

    Args:
        action (callable): A function of the form
            ``func(req, resp, resource)``, where `resource` is a
            reference to the resource class instance associated with the
            request

    """

    def _after(responder_or_resource):
        if isinstance(responder_or_resource, six.class_types):
            resource = responder_or_resource

            for method in HTTP_METHODS:
                responder_name = 'on_' + method.lower()

                try:
                    responder = getattr(resource, responder_name)
                except AttributeError:
                    # resource does not implement this method
                    pass
                else:
                    # Usually expect a method, but any callable will do
                    if callable(responder):

                        def let(responder=responder):
                            do_after_all = _wrap_with_after(action, responder)

                            setattr(resource, responder_name, do_after_all)

                        let()

            return resource

        else:
            responder = responder_or_resource
            do_after_one = _wrap_with_after(action, responder)

            return do_after_one

    return _after


# -----------------------------------------------------------------------------
# Helpers
# -----------------------------------------------------------------------------


def _wrap_with_after(action, responder):
    """Execute the given action function after a responder method.

    Args:
        action: A function with a signature similar to a resource responder
            method, taking the form ``func(req, resp, resource)``.
        responder: The responder method to wrap.
    """

    # NOTE(swistakm): create shim before checking what will be actually
    # decorated. This helps to avoid excessive nesting
    if 'resource' in get_argnames(action):
        shim = action
    else:
        # TODO(kgriffs): This decorator does not work on callable
        # classes in Python vesions prior to 3.4.
        #
        # @wraps(action)
        def shim(req, resp, resource):
            action(req, resp)

    @wraps(responder)
    def do_after(self, req, resp, **kwargs):
        responder(self, req, resp, **kwargs)
        shim(req, resp, self)

    return do_after


def _wrap_with_before(action, responder):
    """Execute the given action function before a responder method.

    Args:
        action: A function with a similar signature to a resource responder
            method, taking the form ``func(req, resp, resource, params)``.
        responder: The responder method to wrap
    """

    # NOTE(swistakm): create shim before checking what will be actually
    # decorated. This allows to avoid excessive nesting
    if 'resource' in get_argnames(action):
        shim = action
    else:
        # TODO(kgriffs): This decorator does not work on callable
        # classes in Python vesions prior to 3.4.
        #
        # @wraps(action)
        def shim(req, resp, resource, kwargs):
            # NOTE(kgriffs): Don't have to pass "self" even if has_self,
            # since method is assumed to be bound.
            action(req, resp, kwargs)

    @wraps(responder)
    def do_before(self, req, resp, **kwargs):
        shim(req, resp, self, kwargs)
        responder(self, req, resp, **kwargs)

    return do_before



import functools
import json

import falcon
from falcon import testing


# --------------------------------------------------------------------
# Hooks
# --------------------------------------------------------------------


def validate_output(req, resp):
    raise falcon.HTTPError(falcon.HTTP_723, 'Tricky')


def serialize_body(req, resp):
    body = resp.body
    if body is not None:
        resp.body = json.dumps(body)
    else:
        resp.body = 'Nothing to see here. Move along.'


def fluffiness(req, resp):
    resp.body = 'fluffy'


def resource_aware_fluffiness(req, resp, resource):
    assert resource
    fluffiness(req, resp)


class ResourceAwareFluffiness(object):
    def __call__(self, req, resp, resource):
        assert resource
        fluffiness(req, resp)


def cuteness(req, resp):
    if resp.body == 'fluffy':
        resp.body += ' and cute'


def resource_aware_cuteness(req, resp, resource):
    assert resource
    cuteness(req, resp)


class Smartness(object):
    def __call__(self, req, resp):
        if resp.body:
            resp.body += ' and smart'
        else:
            resp.body = 'smart'


# NOTE(kgriffs): Use partial methods for these next two in order
# to make sure we handle that correctly.
def things_in_the_head(header, value, req, resp, params):
    resp.set_header(header, value)


bunnies_in_the_head = functools.partial(things_in_the_head,
                                        'X-Bunnies', 'fluffy')

cuteness_in_the_head = functools.partial(things_in_the_head,
                                         'X-Cuteness', 'cute')


def fluffiness_in_the_head(req, resp):
    resp.set_header('X-Fluffiness', 'fluffy')


def cuteness_in_the_head(req, resp):
    resp.set_header('X-Cuteness', 'cute')


# --------------------------------------------------------------------
# Resources
# --------------------------------------------------------------------


class WrappedRespondersResource(object):

    @falcon.after(serialize_body)
    @falcon.after(validate_output)
    def on_get(self, req, resp):
        self.req = req
        self.resp = resp

    @falcon.after(serialize_body)
    def on_put(self, req, resp):
        self.req = req
        self.resp = resp
        resp.body = {'animal': 'falcon'}

    @falcon.after(Smartness())
    def on_post(self, req, resp):
        pass


@falcon.after(cuteness)
@falcon.after(fluffiness)
class WrappedClassResource(object):

    # Test that the decorator skips non-callables
    on_post = False

    def __init__(self):
        # Test that the decorator skips non-callables
        self.on_patch = []

    def on_get(self, req, resp):
        self.req = req
        self.resp = resp

    @falcon.after(fluffiness_in_the_head)
    @falcon.after(cuteness_in_the_head)
    def on_head(self, req, resp):
        self.req = req
        self.resp = resp


# NOTE(swistakm): we use both type of hooks (class and method)
# at once for the sake of simplicity
@falcon.after(resource_aware_cuteness)
class ClassResourceWithAwareHooks(object):

    # Test that the decorator skips non-callables
    on_post = False

    hook_as_class = ResourceAwareFluffiness()

    def __init__(self):
        # Test that the decorator skips non-callables
        self.on_patch = []

    @falcon.after(resource_aware_fluffiness)
    def on_get(self, req, resp):
        self._capture(req, resp)

    @falcon.after(resource_aware_fluffiness)
    def on_head(self, req, resp):
        self._capture(req, resp)

    @falcon.after(hook_as_class)
    def on_put(self, req, resp):
        self._capture(req, resp)

    @falcon.after(hook_as_class.__call__)
    def on_post(self, req, resp):
        self._capture(req, resp)

    def _capture(self, req, resp):
        self.req = req
        self.resp = resp


# --------------------------------------------------------------------
# Tests
# --------------------------------------------------------------------


class TestHooks(testing.TestCase):

    def setUp(self):
        super(TestHooks, self).setUp()

        self.resource = WrappedRespondersResource()
        self.api.add_route('/', self.resource)

        self.wrapped_resource = WrappedClassResource()
        self.api.add_route('/wrapped', self.wrapped_resource)

        self.wrapped_resource_aware = ClassResourceWithAwareHooks()
        self.api.add_route('/wrapped_aware', self.wrapped_resource_aware)

    def test_output_validator(self):
        result = self.simulate_get()
        self.assertEqual(result.status_code, 723)
        self.assertEqual(result.text, '{\n    "title": "Tricky"\n}')

    def test_serializer(self):
        result = self.simulate_put()
        self.assertEqual('{"animal": "falcon"}', result.text)

    def test_hook_as_callable_class(self):
        result = self.simulate_post()
        self.assertEqual('smart', result.text)

    def test_wrapped_resource(self):
        result = self.simulate_get('/wrapped')
        self.assertEqual(result.status_code, 200)
        self.assertEqual(result.text, 'fluffy and cute', )

        result = self.simulate_head('/wrapped')
        self.assertEqual(result.status_code, 200)
        self.assertEqual(result.headers['X-Fluffiness'], 'fluffy')
        self.assertEqual(result.headers['X-Cuteness'], 'cute')

        result = self.simulate_post('/wrapped')
        self.assertEqual(result.status_code, 405)

        result = self.simulate_patch('/wrapped')
        self.assertEqual(result.status_code, 405)

        # Decorator should not affect the default on_options responder
        result = self.simulate_options('/wrapped')
        self.assertEqual(result.status_code, 204)
        self.assertFalse(result.text)

    def test_wrapped_resource_with_hooks_aware_of_resource(self):
        expected = 'fluffy and cute'

        result = self.simulate_get('/wrapped_aware')
        self.assertEqual(result.status_code, 200)
        self.assertEqual(expected, result.text)

        for test in (self.simulate_head, self.simulate_put, self.simulate_post):
            result = test('/wrapped_aware')
            self.assertEqual(result.status_code, 200)
            self.assertEqual(self.wrapped_resource_aware.resp.body, expected)

        result = self.simulate_patch('/wrapped_aware')
        self.assertEqual(result.status_code, 405)

        # Decorator should not affect the default on_options responder
        result = self.simulate_options('/wrapped_aware')
        self.assertEqual(result.status_code, 204)
        self.assertFalse(result.text)



# -*- coding: utf-8

import falcon
from falcon.http_status import HTTPStatus
import falcon.testing as testing


def before_hook(req, resp, params):
    raise HTTPStatus(falcon.HTTP_200,
                     headers={'X-Failed': 'False'},
                     body='Pass')


def after_hook(req, resp, resource):
    resp.status = falcon.HTTP_200
    resp.set_header('X-Failed', 'False')
    resp.body = 'Pass'


def noop_after_hook(req, resp, resource):
    pass


class TestStatusResource:

    @falcon.before(before_hook)
    def on_get(self, req, resp):
        resp.status = falcon.HTTP_500
        resp.set_header('X-Failed', 'True')
        resp.body = 'Fail'

    def on_post(self, req, resp):
        resp.status = falcon.HTTP_500
        resp.set_header('X-Failed', 'True')
        resp.body = 'Fail'

        raise HTTPStatus(falcon.HTTP_200,
                         headers={'X-Failed': 'False'},
                         body='Pass')

    @falcon.after(after_hook)
    def on_put(self, req, resp):
        resp.status = falcon.HTTP_500
        resp.set_header('X-Failed', 'True')
        resp.body = 'Fail'

    def on_patch(self, req, resp):
        raise HTTPStatus(falcon.HTTP_200,
                         body=None)

    @falcon.after(noop_after_hook)
    def on_delete(self, req, resp):
        raise HTTPStatus(falcon.HTTP_200,
                         headers={'X-Failed': 'False'},
                         body='Pass')


class TestHookResource:

    def on_get(self, req, resp):
        resp.status = falcon.HTTP_500
        resp.set_header('X-Failed', 'True')
        resp.body = 'Fail'

    def on_patch(self, req, resp):
        raise HTTPStatus(falcon.HTTP_200,
                         body=None)

    def on_delete(self, req, resp):
        raise HTTPStatus(falcon.HTTP_200,
                         headers={'X-Failed': 'False'},
                         body='Pass')


class TestHTTPStatus(testing.TestBase):
    def before(self):
        self.resource = TestStatusResource()
        self.api.add_route('/status', self.resource)

    def test_raise_status_in_before_hook(self):
        """ Make sure we get the 200 raised by before hook """
        body = self.simulate_request('/status', method='GET', decode='utf-8')
        self.assertEqual(self.srmock.status, falcon.HTTP_200)
        self.assertIn(('x-failed', 'False'), self.srmock.headers)
        self.assertEqual(body, 'Pass')

    def test_raise_status_in_responder(self):
        """ Make sure we get the 200 raised by responder """
        body = self.simulate_request('/status', method='POST', decode='utf-8')
        self.assertEqual(self.srmock.status, falcon.HTTP_200)
        self.assertIn(('x-failed', 'False'), self.srmock.headers)
        self.assertEqual(body, 'Pass')

    def test_raise_status_runs_after_hooks(self):
        """ Make sure after hooks still run """
        body = self.simulate_request('/status', method='PUT', decode='utf-8')
        self.assertEqual(self.srmock.status, falcon.HTTP_200)
        self.assertIn(('x-failed', 'False'), self.srmock.headers)
        self.assertEqual(body, 'Pass')

    def test_raise_status_survives_after_hooks(self):
        """ Make sure after hook doesn't overwrite our status """
        body = self.simulate_request('/status', method='DELETE',
                                     decode='utf-8')
        self.assertEqual(self.srmock.status, falcon.HTTP_200)
        self.assertIn(('x-failed', 'False'), self.srmock.headers)
        self.assertEqual(body, 'Pass')

    def test_raise_status_empty_body(self):
        """ Make sure passing None to body results in empty body """
        body = self.simulate_request('/status', method='PATCH', decode='utf-8')
        self.assertEqual(body, '')


class TestHTTPStatusWithMiddleware(testing.TestBase):
    def before(self):
        self.resource = TestHookResource()

    def test_raise_status_in_process_request(self):
        """ Make sure we can raise status from middleware process request """
        class TestMiddleware:
            def process_request(self, req, resp):
                raise HTTPStatus(falcon.HTTP_200,
                                 headers={'X-Failed': 'False'},
                                 body='Pass')

        self.api = falcon.API(middleware=TestMiddleware())
        self.api.add_route('/status', self.resource)

        body = self.simulate_request('/status', method='GET', decode='utf-8')
        self.assertEqual(self.srmock.status, falcon.HTTP_200)
        self.assertIn(('x-failed', 'False'), self.srmock.headers)
        self.assertEqual(body, 'Pass')

    def test_raise_status_in_process_resource(self):
        """ Make sure we can raise status from middleware process resource """
        class TestMiddleware:
            def process_resource(self, req, resp, resource, params):
                raise HTTPStatus(falcon.HTTP_200,
                                 headers={'X-Failed': 'False'},
                                 body='Pass')

        self.api = falcon.API(middleware=TestMiddleware())
        self.api.add_route('/status', self.resource)

        body = self.simulate_request('/status', method='GET', decode='utf-8')
        self.assertEqual(self.srmock.status, falcon.HTTP_200)
        self.assertIn(('x-failed', 'False'), self.srmock.headers)
        self.assertEqual(body, 'Pass')

    def test_raise_status_runs_process_response(self):
        """ Make sure process_response still runs """
        class TestMiddleware:
            def process_response(self, req, resp, response):
                resp.status = falcon.HTTP_200
                resp.set_header('X-Failed', 'False')
                resp.body = 'Pass'

        self.api = falcon.API(middleware=TestMiddleware())
        self.api.add_route('/status', self.resource)

        body = self.simulate_request('/status', method='GET', decode='utf-8')
        self.assertEqual(self.srmock.status, falcon.HTTP_200)
        self.assertIn(('x-failed', 'False'), self.srmock.headers)
        self.assertEqual(body, 'Pass')



import io

import ddt
import six

import falcon
import falcon.testing as testing


# NOTE(kgriffs): Concept from Gunicorn's source (wsgi.py)
class FileWrapper(object):

    def __init__(self, file_like, block_size=8192):
        self.file_like = file_like
        self.block_size = block_size

    def __getitem__(self, key):
        data = self.file_like.read(self.block_size)
        if data:
            return data

        raise IndexError


class HelloResource(object):
    sample_status = '200 OK'
    sample_unicode = (u'Hello World! \x80' +
                      six.text_type(testing.rand_string(0, 0)))

    sample_utf8 = sample_unicode.encode('utf-8')

    def __init__(self, mode):
        self.called = False
        self.mode = mode

    def on_get(self, req, resp):
        self.called = True
        self.req, self.resp = req, resp

        resp.status = falcon.HTTP_200

        if 'stream' in self.mode:
            if 'filelike' in self.mode:
                stream = io.BytesIO(self.sample_utf8)
            else:
                stream = [self.sample_utf8]

            if 'stream_len' in self.mode:
                stream_len = len(self.sample_utf8)
            else:
                stream_len = None

            if 'use_helper' in self.mode:
                resp.set_stream(stream, stream_len)
            else:
                resp.stream = stream
                resp.stream_len = stream_len

        if 'body' in self.mode:
            if 'bytes' in self.mode:
                resp.body = self.sample_utf8
            else:
                resp.body = self.sample_unicode

        if 'data' in self.mode:
            resp.data = self.sample_utf8

    def on_head(self, req, resp):
        self.on_get(req, resp)


class NoStatusResource(object):
    def on_get(self, req, resp):
        pass


@ddt.ddt
class TestHelloWorld(testing.TestCase):

    def setUp(self):
        super(TestHelloWorld, self).setUp()

    def test_env_headers_list_of_tuples(self):
        env = testing.create_environ(headers=[('User-Agent', 'Falcon-Test')])
        self.assertEqual(env['HTTP_USER_AGENT'], 'Falcon-Test')

    def test_root_route(self):
        doc = {u'message': u'Hello world!'}
        resource = testing.SimpleTestResource(json=doc)
        self.api.add_route('/', resource)

        result = self.simulate_get()
        self.assertEqual(result.json, doc)

    def test_no_route(self):
        result = self.simulate_get('/seenoevil')
        self.assertEqual(result.status_code, 404)

    @ddt.data(
        ('/body', HelloResource('body'), lambda r: r.body.encode('utf-8')),
        ('/bytes', HelloResource('body, bytes'), lambda r: r.body),
        ('/data', HelloResource('data'), lambda r: r.data),
    )
    @ddt.unpack
    def test_body(self, path, resource, get_body):
        self.api.add_route(path, resource)

        result = self.simulate_get(path)
        resp = resource.resp

        content_length = int(result.headers['content-length'])
        self.assertEqual(content_length, len(resource.sample_utf8))

        self.assertEqual(result.status, resource.sample_status)
        self.assertEqual(resp.status, resource.sample_status)
        self.assertEqual(get_body(resp), resource.sample_utf8)
        self.assertEqual(result.content, resource.sample_utf8)

    def test_no_body_on_head(self):
        self.api.add_route('/body', HelloResource('body'))
        result = self.simulate_head('/body')

        self.assertFalse(result.content)
        self.assertEqual(result.status_code, 200)

    def test_stream_chunked(self):
        resource = HelloResource('stream')
        self.api.add_route('/chunked-stream', resource)

        result = self.simulate_get('/chunked-stream')

        self.assertEqual(result.content, resource.sample_utf8)
        self.assertNotIn('content-length', result.headers)

    def test_stream_known_len(self):
        resource = HelloResource('stream, stream_len')
        self.api.add_route('/stream', resource)

        result = self.simulate_get('/stream')
        self.assertTrue(resource.called)

        expected_len = resource.resp.stream_len
        actual_len = int(result.headers['content-length'])
        self.assertEqual(actual_len, expected_len)
        self.assertEqual(len(result.content), expected_len)
        self.assertEqual(result.content, resource.sample_utf8)

    def test_filelike(self):
        resource = HelloResource('stream, stream_len, filelike')
        self.api.add_route('/filelike', resource)

        for file_wrapper in (None, FileWrapper):
            result = self.simulate_get('/filelike', file_wrapper=file_wrapper)
            self.assertTrue(resource.called)

            expected_len = resource.resp.stream_len
            actual_len = int(result.headers['content-length'])
            self.assertEqual(actual_len, expected_len)
            self.assertEqual(len(result.content), expected_len)

    def test_filelike_using_helper(self):
        resource = HelloResource('stream, stream_len, filelike, use_helper')
        self.api.add_route('/filelike-helper', resource)

        result = self.simulate_get('/filelike-helper')
        self.assertTrue(resource.called)

        expected_len = resource.resp.stream_len
        actual_len = int(result.headers['content-length'])
        self.assertEqual(actual_len, expected_len)
        self.assertEqual(len(result.content), expected_len)

    def test_status_not_set(self):
        self.api.add_route('/nostatus', NoStatusResource())

        result = self.simulate_get('/nostatus')

        self.assertFalse(result.content)
        self.assertEqual(result.status_code, 200)



# Copyright 2013 by Rackspace Hosting, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Default responder implementations."""

from falcon.errors import HTTPBadRequest
from falcon.errors import HTTPNotFound
from falcon.errors import HTTPMethodNotAllowed
from falcon.status_codes import HTTP_204


def path_not_found(req, resp, **kwargs):
    """Raise 404 HTTPNotFound error"""
    raise HTTPNotFound()


def bad_request(req, resp, **kwargs):
    """Raise 400 HTTPBadRequest error"""
    raise HTTPBadRequest('Bad request', 'Invalid HTTP method')


def create_method_not_allowed(allowed_methods):
    """Creates a responder for "405 Method Not Allowed"

    Args:
        allowed_methods: A list of HTTP methods (uppercase) that should be
            returned in the Allow header.

    """
    def method_not_allowed(req, resp, **kwargs):
        """Raise 405 HTTPMethodNotAllowed error"""
        raise HTTPMethodNotAllowed(allowed_methods)

    return method_not_allowed


def create_default_options(allowed_methods):
    """Creates a default responder for the OPTIONS method

    Args:
        allowed_methods: A list of HTTP methods (uppercase) that should be
            returned in the Allow header.

    """
    allowed = ', '.join(allowed_methods)

    def on_options(req, resp, **kwargs):
        resp.status = HTTP_204
        resp.set_header('Allow', allowed)
        resp.set_header('Content-Length', '0')

    return on_options




import falcon
import falcon.testing as testing


class TestResponseBody(testing.TestBase):

    def test_append_body(self):
        text = 'Hello beautiful world! '
        resp = falcon.Response()
        resp.body = ''

        for token in text.split():
            resp.body += token
            resp.body += ' '

        self.assertEqual(resp.body, text)



from falcon.request import Request
import falcon.testing as testing


def test_remote_addr_only():
    req = Request(testing.create_environ(
        host='example.com',
        path='/access_route',
        headers={
            'Forwarded': ('for=192.0.2.43, for="[2001:db8:cafe::17]:555",'
                          'for="unknown", by=_hidden,for="\\"\\\\",'
                          'for="198\\.51\\.100\\.17\\:1236";'
                          'proto=https;host=example.com')
        }))

    assert req.remote_addr == '127.0.0.1'


def test_rfc_forwarded():
    req = Request(testing.create_environ(
        host='example.com',
        path='/access_route',
        headers={
            'Forwarded': ('for=192.0.2.43,for=,'
                          'for="[2001:db8:cafe::17]:555",'
                          'for=x,'
                          'for="unknown", by=_hidden,for="\\"\\\\",'
                          'for="_don\\\"t_\\try_this\\\\at_home_\\42",'
                          'for="198\\.51\\.100\\.17\\:1236";'
                          'proto=https;host=example.com')
        }))

    compares = ['192.0.2.43', '2001:db8:cafe::17', 'x',
                'unknown', '"\\', '_don"t_try_this\\at_home_42',
                '198.51.100.17']

    req.access_route == compares

    # test cached
    req.access_route == compares


def test_malformed_rfc_forwarded():
    req = Request(testing.create_environ(
        host='example.com',
        path='/access_route',
        headers={
            'Forwarded': 'for'
        }))

    req.access_route == []

    # test cached
    req.access_route == []


def test_x_forwarded_for():
    req = Request(testing.create_environ(
        host='example.com',
        path='/access_route',
        headers={
            'X-Forwarded-For': ('192.0.2.43, 2001:db8:cafe::17,'
                                'unknown, _hidden, 203.0.113.60')
        }))

    assert req.access_route == [
        '192.0.2.43',
        '2001:db8:cafe::17',
        'unknown',
        '_hidden',
        '203.0.113.60'
    ]


def test_x_real_ip():
    req = Request(testing.create_environ(
        host='example.com',
        path='/access_route',
        headers={
            'X-Real-IP': '2001:db8:cafe::17'
        }))

    assert req.access_route == ['2001:db8:cafe::17']


def test_remote_addr():
    req = Request(testing.create_environ(
        host='example.com',
        path='/access_route'))

    assert req.access_route == ['127.0.0.1']


def test_remote_addr_missing():
    env = testing.create_environ(host='example.com', path='/access_route')
    del env['REMOTE_ADDR']

    req = Request(env)
    assert req.access_route == []



from collections import defaultdict
from datetime import datetime

import ddt
import six

import falcon
from falcon import testing


class XmlResource(object):
    def __init__(self, content_type):
        self.content_type = content_type

    def on_get(self, req, resp):
        resp.set_header('content-type', self.content_type)


class HeaderHelpersResource(object):

    def __init__(self, last_modified=None):
        if last_modified is not None:
            self.last_modified = last_modified
        else:
            self.last_modified = datetime.utcnow()

    def _overwrite_headers(self, req, resp):
        resp.content_type = 'x-falcon/peregrine'
        resp.cache_control = ['no-store']

    def on_get(self, req, resp):
        resp.body = '{}'
        resp.content_type = 'x-falcon/peregrine'
        resp.cache_control = [
            'public', 'private', 'no-cache', 'no-store', 'must-revalidate',
            'proxy-revalidate', 'max-age=3600', 's-maxage=60', 'no-transform'
        ]

        resp.etag = 'fa0d1a60ef6616bb28038515c8ea4cb2'
        resp.last_modified = self.last_modified
        resp.retry_after = 3601

        # Relative URI's are OK per http://goo.gl/DbVqR
        resp.location = '/things/87'
        resp.content_location = '/things/78'

        if req.range_unit is None or req.range_unit == 'bytes':
            # bytes 0-499/10240
            resp.content_range = (0, 499, 10 * 1024)
        else:
            resp.content_range = (0, 25, 100, req.range_unit)

        resp.accept_ranges = 'bytes'

        self.resp = resp

    def on_head(self, req, resp):
        resp.set_header('Content-Type', 'x-swallow/unladen')
        resp.set_header('X-Auth-Token', 'setecastronomy')
        resp.set_header('X-AUTH-TOKEN', 'toomanysecrets')

        resp.location = '/things/87'
        del resp.location

        self._overwrite_headers(req, resp)

        self.resp = resp

    def on_post(self, req, resp):
        resp.set_headers([
            ('CONTENT-TYPE', 'x-swallow/unladen'),
            ('X-Auth-Token', 'setecastronomy'),
            ('X-AUTH-TOKEN', 'toomanysecrets')
        ])

        self._overwrite_headers(req, resp)

        self.resp = resp

    def on_put(self, req, resp):
        resp.set_headers({
            'CONTENT-TYPE': 'x-swallow/unladen',
            'X-aUTH-tOKEN': 'toomanysecrets'
        })

        self._overwrite_headers(req, resp)

        self.resp = resp


class LocationHeaderUnicodeResource(object):

    URL1 = u'/\u00e7runchy/bacon'
    URL2 = u'ab\u00e7' if six.PY3 else 'ab\xc3\xa7'

    def on_get(self, req, resp):
        resp.location = self.URL1
        resp.content_location = self.URL2

    def on_head(self, req, resp):
        resp.location = self.URL2
        resp.content_location = self.URL1


class UnicodeHeaderResource(object):

    def on_get(self, req, resp):
        resp.set_headers([
            (u'X-auTH-toKEN', 'toomanysecrets'),
            ('Content-TYpE', u'application/json'),
            (u'X-symBOl', u'@'),
        ])

    def on_post(self, req, resp):
        resp.set_headers([
            (u'X-symb\u00F6l', 'thing'),
        ])

    def on_put(self, req, resp):
        resp.set_headers([
            ('X-Thing', u'\u00FF'),
        ])


class VaryHeaderResource(object):

    def __init__(self, vary):
        self.vary = vary

    def on_get(self, req, resp):
        resp.body = '{}'
        resp.vary = self.vary


class LinkHeaderResource(object):

    def __init__(self):
        self._links = []

    def add_link(self, *args, **kwargs):
        self._links.append((args, kwargs))

    def on_get(self, req, resp):
        resp.body = '{}'

        for args, kwargs in self._links:
            resp.add_link(*args, **kwargs)


class AppendHeaderResource(object):

    def on_get(self, req, resp):
        resp.append_header('X-Things', 'thing-1')
        resp.append_header('X-THINGS', 'thing-2')
        resp.append_header('x-thiNgs', 'thing-3')

    def on_head(self, req, resp):
        resp.set_header('X-things', 'thing-1')
        resp.append_header('X-THINGS', 'thing-2')
        resp.append_header('x-thiNgs', 'thing-3')

    def on_post(self, req, resp):
        resp.append_header('X-Things', 'thing-1')


@ddt.ddt
class TestHeaders(testing.TestCase):

    def setUp(self):
        super(TestHeaders, self).setUp()

        self.sample_body = testing.rand_string(0, 128 * 1024)
        self.resource = testing.SimpleTestResource(body=self.sample_body)
        self.api.add_route('/', self.resource)

    def test_content_length(self):
        result = self.simulate_get()

        content_length = str(len(self.sample_body))
        self.assertEqual(result.headers['Content-Length'], content_length)

    def test_default_value(self):
        self.simulate_get()

        req = self.resource.captured_req
        value = req.get_header('X-Not-Found') or '876'
        self.assertEqual(value, '876')

    def test_required_header(self):
        self.simulate_get()

        try:
            req = self.resource.captured_req
            req.get_header('X-Not-Found', required=True)
            self.fail('falcon.HTTPMissingHeader not raised')
        except falcon.HTTPMissingHeader as ex:
            self.assertIsInstance(ex, falcon.HTTPBadRequest)
            self.assertEqual(ex.title, 'Missing header value')
            expected_desc = 'The X-Not-Found header is required.'
            self.assertEqual(ex.description, expected_desc)

    @ddt.data(falcon.HTTP_204, falcon.HTTP_304)
    def test_no_content_length(self, status):
        self.api.add_route('/xxx', testing.SimpleTestResource(status=status))

        result = self.simulate_get('/xxx')
        self.assertNotIn('Content-Length', result.headers)
        self.assertFalse(result.content)

    def test_content_header_missing(self):
        environ = testing.create_environ()
        req = falcon.Request(environ)
        for header in ('Content-Type', 'Content-Length'):
            self.assertIs(req.get_header(header), None)

    def test_passthrough_request_headers(self):
        request_headers = {
            'X-Auth-Token': 'Setec Astronomy',
            'Content-Type': 'text/plain; charset=utf-8'
        }
        self.simulate_get(headers=request_headers)

        for name, expected_value in request_headers.items():
            actual_value = self.resource.captured_req.get_header(name)
            self.assertEqual(actual_value, expected_value)

        self.simulate_get(headers=self.resource.captured_req.headers)

        # Compare the request HTTP headers with the original headers
        for name, expected_value in request_headers.items():
            actual_value = self.resource.captured_req.get_header(name)
            self.assertEqual(actual_value, expected_value)

    def test_headers_as_list(self):
        headers = [
            ('Client-ID', '692ba466-74bb-11e3-bf3f-7567c531c7ca'),
            ('Accept', 'audio/*; q=0.2, audio/basic')
        ]

        # Unit test
        environ = testing.create_environ(headers=headers)
        req = falcon.Request(environ)

        for name, value in headers:
            self.assertIn((name.upper(), value), req.headers.items())

        # Functional test
        self.api.add_route('/', testing.SimpleTestResource(headers=headers))
        result = self.simulate_get()

        for name, value in headers:
            self.assertEqual(result.headers[name], value)

    def test_default_media_type(self):
        resource = testing.SimpleTestResource(body='Hello world!')
        self._check_header(resource, 'Content-Type', falcon.DEFAULT_MEDIA_TYPE)

    @ddt.data(
        ('text/plain; charset=UTF-8', u'Hello Unicode! \U0001F638'),

        # NOTE(kgriffs): This only works because the client defaults to
        # ISO-8859-1 IFF the media type is 'text'.
        ('text/plain', 'Hello ISO-8859-1!'),
    )
    @ddt.unpack
    def test_override_default_media_type(self, content_type, body):
        self.api = falcon.API(media_type=content_type)
        self.api.add_route('/', testing.SimpleTestResource(body=body))
        result = self.simulate_get()

        self.assertEqual(result.text, body)
        self.assertEqual(result.headers['Content-Type'], content_type)

    def test_override_default_media_type_missing_encoding(self):
        body = u'{"msg": "Hello Unicode! \U0001F638"}'

        self.api = falcon.API(media_type='application/json')
        self.api.add_route('/', testing.SimpleTestResource(body=body))
        result = self.simulate_get()

        self.assertEqual(result.content, body.encode('utf-8'))
        self.assertIsInstance(result.text, six.text_type)
        self.assertEqual(result.text, body)
        self.assertEqual(result.json, {u'msg': u'Hello Unicode! \U0001F638'})

    def test_response_header_helpers_on_get(self):
        last_modified = datetime(2013, 1, 1, 10, 30, 30)
        resource = HeaderHelpersResource(last_modified)
        self.api.add_route('/', resource)
        result = self.simulate_get()

        resp = resource.resp

        content_type = 'x-falcon/peregrine'
        self.assertEqual(resp.content_type, content_type)
        self.assertEqual(result.headers['Content-Type'], content_type)

        cache_control = ('public, private, no-cache, no-store, '
                         'must-revalidate, proxy-revalidate, max-age=3600, '
                         's-maxage=60, no-transform')

        self.assertEqual(resp.cache_control, cache_control)
        self.assertEqual(result.headers['Cache-Control'], cache_control)

        etag = 'fa0d1a60ef6616bb28038515c8ea4cb2'
        self.assertEqual(resp.etag, etag)
        self.assertEqual(result.headers['Etag'], etag)

        lm_date = 'Tue, 01 Jan 2013 10:30:30 GMT'
        self.assertEqual(resp.last_modified, lm_date)
        self.assertEqual(result.headers['Last-Modified'], lm_date)

        self.assertEqual(resp.retry_after, '3601')
        self.assertEqual(result.headers['Retry-After'], '3601')

        self.assertEqual(resp.location, '/things/87')
        self.assertEqual(result.headers['Location'], '/things/87')

        self.assertEqual(resp.content_location, '/things/78')
        self.assertEqual(result.headers['Content-Location'], '/things/78')

        content_range = 'bytes 0-499/10240'
        self.assertEqual(resp.content_range, content_range)
        self.assertEqual(result.headers['Content-Range'], content_range)

        resp.content_range = (1, 499, 10 * 1024, u'bytes')
        self.assertIsInstance(resp.content_range, str)
        self.assertEqual(resp.content_range, 'bytes 1-499/10240')

        self.assertEqual(resp.accept_ranges, 'bytes')
        self.assertEqual(result.headers['Accept-Ranges'], 'bytes')

        req_headers = {'Range': 'items=0-25'}
        result = self.simulate_get(headers=req_headers)
        self.assertEqual(result.headers['Content-Range'], 'items 0-25/100')

        # Check for duplicate headers
        hist = defaultdict(lambda: 0)
        for name, value in result.headers.items():
            hist[name] += 1
            self.assertEqual(1, hist[name])

    def test_unicode_location_headers(self):
        self.api.add_route('/', LocationHeaderUnicodeResource())

        result = self.simulate_get()
        self.assertEqual(result.headers['Location'], '/%C3%A7runchy/bacon')
        self.assertEqual(result.headers['Content-Location'], 'ab%C3%A7')

        # Test with the values swapped
        result = self.simulate_head()
        self.assertEqual(result.headers['Content-Location'],
                         '/%C3%A7runchy/bacon')
        self.assertEqual(result.headers['Location'], 'ab%C3%A7')

    def test_unicode_headers_convertable(self):
        self.api.add_route('/', UnicodeHeaderResource())

        result = self.simulate_get('/')

        self.assertEqual(result.headers['Content-Type'], 'application/json')
        self.assertEqual(result.headers['X-Auth-Token'], 'toomanysecrets')
        self.assertEqual(result.headers['X-Symbol'], '@')

    def test_unicode_headers_not_convertable(self):
        if six.PY3:
            self.skipTest('Test only applies to Python 2')

        self.api.add_route('/', UnicodeHeaderResource())
        self.assertRaises(UnicodeEncodeError, self.simulate_post, '/')
        self.assertRaises(UnicodeEncodeError, self.simulate_put, '/')

    def test_response_set_and_get_header(self):
        resource = HeaderHelpersResource()
        self.api.add_route('/', resource)

        for method in ('HEAD', 'POST', 'PUT'):
            result = self.simulate_request(method=method)

            content_type = 'x-falcon/peregrine'
            self.assertEqual(result.headers['Content-Type'], content_type)
            self.assertEqual(resource.resp.get_header('content-TyPe'),
                             content_type)

            self.assertEqual(result.headers['Cache-Control'], 'no-store')
            self.assertEqual(result.headers['X-Auth-Token'], 'toomanysecrets')

            self.assertEqual(resource.resp.location, None)
            self.assertEqual(resource.resp.get_header('not-real'), None)

            # Check for duplicate headers
            hist = defaultdict(int)
            for name, value in result.headers.items():
                hist[name] += 1
                self.assertEqual(hist[name], 1)

    def test_response_append_header(self):
        self.api.add_route('/', AppendHeaderResource())

        for method in ('HEAD', 'GET'):
            result = self.simulate_request(method=method)
            value = result.headers['x-things']
            self.assertEqual(value, 'thing-1,thing-2,thing-3')

        result = self.simulate_request(method='POST')
        self.assertEqual(result.headers['x-things'], 'thing-1')

    def test_vary_star(self):
        self.api.add_route('/', VaryHeaderResource(['*']))
        result = self.simulate_get()
        self.assertEqual(result.headers['vary'], '*')

    @ddt.data(
        (['accept-encoding'], 'accept-encoding'),
        ([u'accept-encoding', 'x-auth-token'], 'accept-encoding, x-auth-token'),
        (('accept-encoding', u'x-auth-token'), 'accept-encoding, x-auth-token'),
    )
    @ddt.unpack
    def test_vary_header(self, vary, expected_value):
        resource = VaryHeaderResource(vary)
        self._check_header(resource, 'Vary', expected_value)

    def test_content_type_no_body(self):
        self.api.add_route('/', testing.SimpleTestResource())
        result = self.simulate_get()

        # NOTE(kgriffs): Even when there is no body, Content-Type
        # should still be included per wsgiref.validate
        self.assertIn('Content-Type', result.headers)
        self.assertEqual(result.headers['Content-Length'], '0')

    @ddt.data(falcon.HTTP_204, falcon.HTTP_304)
    def test_no_content_type(self, status):
        self.api.add_route('/', testing.SimpleTestResource(status=status))

        result = self.simulate_get()
        self.assertNotIn('Content-Type', result.headers)

    def test_custom_content_type(self):
        content_type = 'application/xml; charset=utf-8'
        resource = XmlResource(content_type)
        self._check_header(resource, 'Content-Type', content_type)

    def test_add_link_single(self):
        expected_value = '</things/2842>; rel=next'

        resource = LinkHeaderResource()
        resource.add_link('/things/2842', 'next')

        self._check_link_header(resource, expected_value)

    def test_add_link_multiple(self):
        expected_value = (
            '</things/2842>; rel=next, ' +
            '<http://%C3%A7runchy/bacon>; rel=contents, ' +
            '<ab%C3%A7>; rel="http://example.com/ext-type", ' +
            '<ab%C3%A7>; rel="http://example.com/%C3%A7runchy", ' +
            '<ab%C3%A7>; rel="https://example.com/too-%C3%A7runchy", ' +
            '</alt-thing>; rel="alternate http://example.com/%C3%A7runchy"')

        uri = u'ab\u00e7' if six.PY3 else 'ab\xc3\xa7'

        resource = LinkHeaderResource()
        resource.add_link('/things/2842', 'next')
        resource.add_link(u'http://\u00e7runchy/bacon', 'contents')
        resource.add_link(uri, 'http://example.com/ext-type')
        resource.add_link(uri, u'http://example.com/\u00e7runchy')
        resource.add_link(uri, u'https://example.com/too-\u00e7runchy')
        resource.add_link('/alt-thing',
                          u'alternate http://example.com/\u00e7runchy')

        self._check_link_header(resource, expected_value)

    def test_add_link_with_title(self):
        expected_value = ('</related/thing>; rel=item; '
                          'title="A related thing"')

        resource = LinkHeaderResource()
        resource.add_link('/related/thing', 'item',
                          title='A related thing')

        self._check_link_header(resource, expected_value)

    def test_add_link_with_title_star(self):
        expected_value = ('</related/thing>; rel=item; '
                          "title*=UTF-8''A%20related%20thing, "
                          '</%C3%A7runchy/thing>; rel=item; '
                          "title*=UTF-8'en'A%20%C3%A7runchy%20thing")

        resource = LinkHeaderResource()
        resource.add_link('/related/thing', 'item',
                          title_star=('', 'A related thing'))

        resource.add_link(u'/\u00e7runchy/thing', 'item',
                          title_star=('en', u'A \u00e7runchy thing'))

        self._check_link_header(resource, expected_value)

    def test_add_link_with_anchor(self):
        expected_value = ('</related/thing>; rel=item; '
                          'anchor="/some%20thing/or-other"')

        resource = LinkHeaderResource()
        resource.add_link('/related/thing', 'item',
                          anchor='/some thing/or-other')

        self._check_link_header(resource, expected_value)

    def test_add_link_with_hreflang(self):
        expected_value = ('</related/thing>; rel=about; '
                          'hreflang=en')

        resource = LinkHeaderResource()
        resource.add_link('/related/thing', 'about', hreflang='en')

        self._check_link_header(resource, expected_value)

    def test_add_link_with_hreflang_multi(self):
        expected_value = ('</related/thing>; rel=about; '
                          'hreflang=en-GB; hreflang=de')

        resource = LinkHeaderResource()
        resource.add_link('/related/thing', 'about',
                          hreflang=('en-GB', 'de'))

        self._check_link_header(resource, expected_value)

    def test_add_link_with_type_hint(self):
        expected_value = ('</related/thing>; rel=alternate; '
                          'type="video/mp4; codecs=avc1.640028"')

        resource = LinkHeaderResource()
        resource.add_link('/related/thing', 'alternate',
                          type_hint='video/mp4; codecs=avc1.640028')

        self._check_link_header(resource, expected_value)

    def test_add_link_complex(self):
        expected_value = ('</related/thing>; rel=alternate; '
                          'title="A related thing"; '
                          "title*=UTF-8'en'A%20%C3%A7runchy%20thing; "
                          'type="application/json"; '
                          'hreflang=en-GB; hreflang=de')

        resource = LinkHeaderResource()
        resource.add_link('/related/thing', 'alternate',
                          title='A related thing',
                          hreflang=('en-GB', 'de'),
                          type_hint='application/json',
                          title_star=('en', u'A \u00e7runchy thing'))

        self._check_link_header(resource, expected_value)

    def test_content_length_options(self):
        result = self.simulate_options()

        content_length = '0'
        self.assertEqual(result.headers['Content-Length'], content_length)

    # ----------------------------------------------------------------------
    # Helpers
    # ----------------------------------------------------------------------

    def _check_link_header(self, resource, expected_value):
        self._check_header(resource, 'Link', expected_value)

    def _check_header(self, resource, header, expected_value):
        self.api.add_route('/', resource)

        result = self.simulate_get()
        self.assertEqual(result.headers[header], expected_value)



# Copyright 2013 by Rackspace Hosting, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Falcon API class."""

import re

import six

from falcon import api_helpers as helpers, DEFAULT_MEDIA_TYPE, routing
from falcon.http_error import HTTPError
from falcon.http_status import HTTPStatus
from falcon.request import Request, RequestOptions
import falcon.responders
from falcon.response import Response
import falcon.status_codes as status
from falcon.util.misc import get_argnames


class API(object):
    """This class is the main entry point into a Falcon-based app.

    Each API instance provides a callable WSGI interface and a routing engine.

    Args:
        media_type (str, optional): Default media type to use as the value for
            the Content-Type header on responses (default 'application/json').
        middleware(object or list, optional): One or more objects
            (instantiated classes) that implement the following middleware
            component interface::

                class ExampleComponent(object):
                    def process_request(self, req, resp):
                        \"\"\"Process the request before routing it.

                        Args:
                            req: Request object that will eventually be
                                routed to an on_* responder method.
                            resp: Response object that will be routed to
                                the on_* responder.
                        \"\"\"

                    def process_resource(self, req, resp, resource, params):
                        \"\"\"Process the request and resource *after* routing.

                        Note:
                            This method is only called when the request matches
                            a route to a resource.

                        Args:
                            req: Request object that will be passed to the
                                routed responder.
                            resp: Response object that will be passed to the
                                responder.
                            resource: Resource object to which the request was
                                routed. May be None if no route was found for
                                the request.
                            params: A dict-like object representing any
                                additional params derived from the route's URI
                                template fields, that will be passed to the
                                resource's responder method as keyword
                                arguments.
                        \"\"\"

                    def process_response(self, req, resp, resource, req_succeeded)
                        \"\"\"Post-processing of the response (after routing).

                        Args:
                            req: Request object.
                            resp: Response object.
                            resource: Resource object to which the request was
                                routed. May be None if no route was found
                                for the request.
                            req_succeeded: True if no exceptions were raised
                                while the framework processed and routed the
                                request; otherwise False.
                        \"\"\"

            See also :ref:`Middleware <middleware>`.

        request_type (Request, optional): ``Request``-like class to use instead
            of Falcon's default class. Among other things, this feature
            affords inheriting from ``falcon.request.Request`` in order
            to override the ``context_type`` class variable.
            (default ``falcon.request.Request``)

        response_type (Response, optional): ``Response``-like class to use
            instead of Falcon's default class. (default
            ``falcon.response.Response``)

        router (object, optional): An instance of a custom router
            to use in lieu of the default engine.
            See also: :ref:`Routing <routing>`.

    Attributes:
        req_options: A set of behavioral options related to incoming
            requests. See also: :py:class:`~.RequestOptions`
    """

    # PERF(kgriffs): Reference via self since that is faster than
    # module global...
    _BODILESS_STATUS_CODES = set([
        status.HTTP_100,
        status.HTTP_101,
        status.HTTP_204,
        status.HTTP_304
    ])

    _STREAM_BLOCK_SIZE = 8 * 1024  # 8 KiB

    __slots__ = ('_request_type', '_response_type',
                 '_error_handlers', '_media_type', '_router', '_sinks',
                 '_serialize_error', 'req_options', '_middleware')

    def __init__(self, media_type=DEFAULT_MEDIA_TYPE,
                 request_type=Request, response_type=Response,
                 middleware=None, router=None):
        self._sinks = []
        self._media_type = media_type

        # set middleware
        self._middleware = helpers.prepare_middleware(middleware)

        self._router = router or routing.DefaultRouter()

        self._request_type = request_type
        self._response_type = response_type

        self._error_handlers = []
        self._serialize_error = helpers.default_serialize_error
        self.req_options = RequestOptions()

        # NOTE(kgriffs): Add default error handlers
        self.add_error_handler(falcon.HTTPError, self._http_error_handler)
        self.add_error_handler(falcon.HTTPStatus, self._http_status_handler)

    def __call__(self, env, start_response):  # noqa: C901
        """WSGI `app` method.

        Makes instances of API callable from a WSGI server. May be used to
        host an API or called directly in order to simulate requests when
        testing the API.

        See also PEP 3333.

        Args:
            env (dict): A WSGI environment dictionary
            start_response (callable): A WSGI helper function for setting
                status and headers on a response.

        """

        req = self._request_type(env, options=self.req_options)
        resp = self._response_type()
        resource = None
        params = {}

        mw_pr_stack = []  # Keep track of executed middleware components
        req_succeeded = False

        try:
            try:
                # NOTE(ealogar): The execution of request middleware
                # should be before routing. This will allow request mw
                # to modify the path.
                for component in self._middleware:
                    process_request, _, process_response = component
                    if process_request is not None:
                        process_request(req, resp)

                    if process_response is not None:
                        mw_pr_stack.append(process_response)

                # NOTE(warsaw): Moved this to inside the try except
                # because it is possible when using object-based
                # traversal for _get_responder() to fail.  An example is
                # a case where an object does not have the requested
                # next-hop child resource. In that case, the object
                # being asked to dispatch to its child will raise an
                # HTTP exception signalling the problem, e.g. a 404.
                responder, params, resource, req.uri_template = self._get_responder(req)
            except Exception as ex:
                if not self._handle_exception(ex, req, resp, params):
                    raise
            else:
                try:
                    # NOTE(kgriffs): If the request did not match any
                    # route, a default responder is returned and the
                    # resource is None. In that case, we skip the
                    # resource middleware methods.
                    if resource is not None:
                        # Call process_resource middleware methods.
                        for component in self._middleware:
                            _, process_resource, _ = component
                            if process_resource is not None:
                                process_resource(req, resp, resource, params)

                    responder(req, resp, **params)
                    req_succeeded = True
                except Exception as ex:
                    if not self._handle_exception(ex, req, resp, params):
                        raise
        finally:
            # NOTE(kgriffs): It may not be useful to still execute
            # response middleware methods in the case of an unhandled
            # exception, but this is done for the sake of backwards
            # compatibility, since it was incidentally the behavior in
            # the 1.0 release before this section of the code was
            # reworked.

            # Call process_response middleware methods.
            while mw_pr_stack:
                process_response = mw_pr_stack.pop()
                try:
                    process_response(req, resp, resource, req_succeeded)
                except Exception as ex:
                    if not self._handle_exception(ex, req, resp, params):
                        raise

                    req_succeeded = False

        #
        # Set status and headers
        #
        if req.method == 'HEAD' or resp.status in self._BODILESS_STATUS_CODES:
            body = []
        else:
            body, length = self._get_body(resp, env.get('wsgi.file_wrapper'))
            if length is not None:
                resp._headers['content-length'] = str(length)

        # NOTE(kgriffs): Based on wsgiref.validate's interpretation of
        # RFC 2616, as commented in that module's source code. The
        # presence of the Content-Length header is not similarly
        # enforced.
        if resp.status in (status.HTTP_204, status.HTTP_304):
            media_type = None
        else:
            media_type = self._media_type

        headers = resp._wsgi_headers(media_type)

        # Return the response per the WSGI spec
        start_response(resp.status, headers)
        return body

    def add_route(self, uri_template, resource, *args, **kwargs):
        """Associates a templatized URI path with a resource.

        Note:
            The following information describes the behavior of
            Falcon's default router.

        A resource is an instance of a class that defines various
        "responder" methods, one for each HTTP method the resource
        allows. Responder names start with `on_` and are named according to
        which HTTP method they handle, as in `on_get`, `on_post`, `on_put`,
        etc.

        If your resource does not support a particular
        HTTP method, simply omit the corresponding responder and
        Falcon will reply with "405 Method not allowed" if that
        method is ever requested.

        Responders must always define at least two arguments to receive
        request and response objects, respectively. For example::

            def on_post(self, req, resp):
                pass

        In addition, if the route's template contains field
        expressions, any responder that desires to receive requests
        for that route must accept arguments named after the respective
        field names defined in the template. A field expression consists
        of a bracketed field name.

        Note:
            Since field names correspond to argument names in responder
            methods, they must be valid Python identifiers.

        For example, given the following template::

            /user/{name}

        A PUT request to "/user/kgriffs" would be routed to::

            def on_put(self, req, resp, name):
                pass

        Individual path segments may contain one or more field
        expressions::

            /repos/{org}/{repo}/compare/{usr0}:{branch0}...{usr1}:{branch1}

        Args:
            uri_template (str): A templatized URI. Care must be
                taken to ensure the template does not mask any sink
                patterns, if any are registered (see also `add_sink`).
            resource (instance): Object which represents a REST
                resource. Falcon will pass "GET" requests to on_get,
                "PUT" requests to on_put, etc. If any HTTP methods are not
                supported by your resource, simply don't define the
                corresponding request handlers, and Falcon will do the right
                thing.

        Note:
            Any additional args and kwargs not defined above are passed
            through to the underlying router's ``add_route()`` method. The
            default router does not expect any additional arguments, but
            custom routers may take advantage of this feature to receive
            additional options when setting up routes.

        """

        # NOTE(richardolsson): Doing the validation here means it doesn't have
        # to be duplicated in every future router implementation.
        if not isinstance(uri_template, six.string_types):
            raise TypeError('uri_template is not a string')

        if not uri_template.startswith('/'):
            raise ValueError("uri_template must start with '/'")

        if '//' in uri_template:
            raise ValueError("uri_template may not contain '//'")

        method_map = routing.create_http_method_map(resource)
        self._router.add_route(uri_template, method_map, resource, *args,
                               **kwargs)

    def add_sink(self, sink, prefix=r'/'):
        """Registers a sink method for the API.

        If no route matches a request, but the path in the requested URI
        matches a sink prefix, Falcon will pass control to the
        associated sink, regardless of the HTTP method requested.

        Using sinks, you can drain and dynamically handle a large number
        of routes, when creating static resources and responders would be
        impractical. For example, you might use a sink to create a smart
        proxy that forwards requests to one or more backend services.

        Args:
            sink (callable): A callable taking the form ``func(req, resp)``.

            prefix (str): A regex string, typically starting with '/', which
                will trigger the sink if it matches the path portion of the
                request's URI. Both strings and precompiled regex objects
                may be specified. Characters are matched starting at the
                beginning of the URI path.

                Note:
                    Named groups are converted to kwargs and passed to
                    the sink as such.

                Warning:
                    If the prefix overlaps a registered route template,
                    the route will take precedence and mask the sink
                    (see also `add_route`).

        """

        if not hasattr(prefix, 'match'):
            # Assume it is a string
            prefix = re.compile(prefix)

        # NOTE(kgriffs): Insert at the head of the list such that
        # in the case of a duplicate prefix, the last one added
        # is preferred.
        self._sinks.insert(0, (prefix, sink))

    def add_error_handler(self, exception, handler=None):
        """Registers a handler for a given exception error type.

        A handler can raise an instance of ``HTTPError`` or
        ``HTTPStatus`` to communicate information about the issue to
        the client.  Alternatively, a handler may modify `resp`
        directly.

        Error handlers are matched in LIFO order. In other words, when
        searching for an error handler to match a raised exception, and
        more than one handler matches the exception type, the framework
        will choose the one that was most recently registered.
        Therefore, more general error handlers (e.g., for the
        ``Exception`` type) should be added first, to avoid masking more
        specific handlers for subclassed types.

        Args:
            exception (type): Whenever an error occurs when handling a request
                that is an instance of this exception class, the associated
                handler will be called.
            handler (callable): A function or callable object taking the form
                ``func(ex, req, resp, params)``.

                If not specified explicitly, the handler will default to
                ``exception.handle``, where ``exception`` is the error
                type specified above, and ``handle`` is a static method
                (i.e., decorated with @staticmethod) that accepts
                the same params just described. For example::

                    class CustomException(CustomBaseException):

                        @staticmethod
                        def handle(ex, req, resp, params):
                            # TODO: Log the error
                            # Convert to an instance of falcon.HTTPError
                            raise falcon.HTTPError(falcon.HTTP_792)


        """

        if handler is None:
            try:
                handler = exception.handle
            except AttributeError:
                raise AttributeError('handler must either be specified '
                                     'explicitly or defined as a static'
                                     'method named "handle" that is a '
                                     'member of the given exception class.')

        # Insert at the head of the list in case we get duplicate
        # adds (will cause the most recently added one to win).
        self._error_handlers.insert(0, (exception, handler))

    def set_error_serializer(self, serializer):
        """Override the default serializer for instances of HTTPError.

        When a responder raises an instance of HTTPError, Falcon converts
        it to an HTTP response automatically. The default serializer
        supports JSON and XML, but may be overridden by this method to
        use a custom serializer in order to support other media types.

        The ``falcon.HTTPError`` class contains helper methods, such as
        `to_json()` and `to_dict()`, that can be used from within
        custom serializers. For example::

            def my_serializer(req, resp, exception):
                representation = None

                preferred = req.client_prefers(('application/x-yaml',
                                                'application/json'))

                if preferred is not None:
                    if preferred == 'application/json':
                        representation = exception.to_json()
                    else:
                        representation = yaml.dump(exception.to_dict(),
                                                   encoding=None)
                    resp.body = representation
                    resp.content_type = preferred

        Note:
            If a custom media type is used and the type includes a
            "+json" or "+xml" suffix, the default serializer will
            convert the error to JSON or XML, respectively. If this
            is not desirable, a custom error serializer may be used
            to override this behavior.

        Args:
            serializer (callable): A function taking the form
                ``func(req, resp, exception)``, where `req` is the request
                object that was passed to the responder method, `resp` is
                the response object, and `exception` is an instance of
                ``falcon.HTTPError``.

        """

        if len(get_argnames(serializer)) == 2:
            serializer = helpers.wrap_old_error_serializer(serializer)

        self._serialize_error = serializer

    # ------------------------------------------------------------------------
    # Helpers that require self
    # ------------------------------------------------------------------------

    def _get_responder(self, req):
        """Searches routes for a matching responder.

        Args:
            req: The request object.

        Returns:
            tuple: A 3-member tuple consisting of a responder callable,
            a ``dict`` containing parsed path fields (if any were specified in
            the matching route's URI template), and a reference to the
            responder's resource instance.

        Note:
            If a responder was matched to the given URI, but the HTTP
            method was not found in the method_map for the responder,
            the responder callable element of the returned tuple will be
            `falcon.responder.bad_request`.

            Likewise, if no responder was matched for the given URI, then
            the responder callable element of the returned tuple will be
            `falcon.responder.path_not_found`
        """

        path = req.path
        method = req.method
        uri_template = None

        route = self._router.find(path)

        if route is not None:
            try:
                resource, method_map, params, uri_template = route
            except ValueError:
                # NOTE(kgriffs): Older routers may not return the
                # template. But for performance reasons they should at
                # least return None if they don't support it.
                resource, method_map, params = route
        else:
            # NOTE(kgriffs): Older routers may indicate that no route
            # was found by returning (None, None, None). Therefore, we
            # normalize resource as the flag to indicate whether or not
            # a route was found, for the sake of backwards-compat.
            resource = None

        if resource is not None:
            try:
                responder = method_map[method]
            except KeyError:
                responder = falcon.responders.bad_request
        else:
            params = {}

            for pattern, sink in self._sinks:
                m = pattern.match(path)
                if m:
                    params = m.groupdict()
                    responder = sink

                    break
            else:
                responder = falcon.responders.path_not_found

        return (responder, params, resource, uri_template)

    def _compose_status_response(self, req, resp, http_status):
        """Composes a response for the given HTTPStatus instance."""

        # PERF(kgriffs): The code to set the status and headers is identical
        # to that used in _compose_error_response(), but refactoring in the
        # name of DRY isn't worth the extra CPU cycles.
        resp.status = http_status.status

        if http_status.headers is not None:
            resp.set_headers(http_status.headers)

        # NOTE(kgriffs): If http_status.body is None, that's OK because
        # it's acceptable to set resp.body to None (to indicate no body).
        resp.body = http_status.body

    def _compose_error_response(self, req, resp, error):
        """Composes a response for the given HTTPError instance."""

        resp.status = error.status

        if error.headers is not None:
            resp.set_headers(error.headers)

        if error.has_representation:
            self._serialize_error(req, resp, error)

    def _http_status_handler(self, status, req, resp, params):
        self._compose_status_response(req, resp, status)

    def _http_error_handler(self, error, req, resp, params):
        self._compose_error_response(req, resp, error)

    def _handle_exception(self, ex, req, resp, params):
        """Handles an exception raised from mw or a responder.

        Args:
            ex: Exception to handle
            req: Current request object to pass to the handler
                registered for the given exception type
            resp: Current response object to pass to the handler
                registered for the given exception type
            params: Responder params to pass to the handler
                registered for the given exception type

        Returns:
            bool: ``True`` if a handler was found and called for the
            exception, ``False`` otherwise.
        """

        for err_type, err_handler in self._error_handlers:
            if isinstance(ex, err_type):
                try:
                    err_handler(ex, req, resp, params)
                except HTTPStatus as status:
                    self._compose_status_response(req, resp, status)
                except HTTPError as error:
                    self._compose_error_response(req, resp, error)

                return True

        # NOTE(kgriffs): No error handlers are defined for ex
        # and it is not one of (HTTPStatus, HTTPError), since it
        # would have matched one of the corresponding default
        # handlers.
        return False

    # PERF(kgriffs): Moved from api_helpers since it is slightly faster
    # to call using self, and this function is called for most
    # requests.
    def _get_body(self, resp, wsgi_file_wrapper=None):
        """Converts resp content into an iterable as required by PEP 333

        Args:
            resp: Instance of falcon.Response
            wsgi_file_wrapper: Reference to wsgi.file_wrapper from the
                WSGI environ dict, if provided by the WSGI server. Used
                when resp.stream is a file-like object (default None).

        Returns:
            tuple: A two-member tuple of the form (iterable, content_length).
            The length is returned as ``None`` when unknown. The
            iterable is determined as follows:

                * If resp.body is not ``None``, returns [resp.body],
                  encoded as UTF-8 if it is a Unicode string.
                  Bytestrings are returned as-is.
                * If resp.data is not ``None``, returns [resp.data]
                * If resp.stream is not ``None``, returns resp.stream
                  iterable using wsgi.file_wrapper, if possible.
                * Otherwise, returns []

        """

        body = resp.body
        if body is not None:
            if not isinstance(body, bytes):
                body = body.encode('utf-8')

            return [body], len(body)

        data = resp.data
        if data is not None:
            return [data], len(data)

        stream = resp.stream
        if stream is not None:
            # NOTE(kgriffs): Heuristic to quickly check if stream is
            # file-like. Not perfect, but should be good enough until
            # proven otherwise.
            if hasattr(stream, 'read'):
                if wsgi_file_wrapper is not None:
                    # TODO(kgriffs): Make block size configurable at the
                    # global level, pending experimentation to see how
                    # useful that would be. See also the discussion on
                    # this GitHub PR: http://goo.gl/XGrtDz
                    iterable = wsgi_file_wrapper(stream,
                                                 self._STREAM_BLOCK_SIZE)
                else:
                    iterable = iter(
                        lambda: stream.read(self._STREAM_BLOCK_SIZE),
                        b''
                    )
            else:
                iterable = stream

            # NOTE(kgriffs): If resp.stream_len is None, content_length
            # will be as well; the caller of _get_body must handle this
            # case by not setting the Content-Length header.
            return iterable, resp.stream_len

        return [], 0



# Copyright 2013 by Rackspace Hosting, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Utilities for the Request class."""

import io


def header_property(wsgi_name):
    """Creates a read-only header property.

    Args:
        wsgi_name (str): Case-sensitive name of the header as it would
            appear in the WSGI environ ``dict`` (i.e., 'HTTP_*')

    Returns:
        A property instance than can be assigned to a class variable.

    """

    def fget(self):
        try:
            return self.env[wsgi_name] or None
        except KeyError:
            return None

    return property(fget)


class BoundedStream(io.IOBase):
    """Wrap *wsgi.input* streams to make them more robust.

    ``socket._fileobject`` and ``io.BufferedReader`` are sometimes used
    to implement *wsgi.input*. However, app developers are often burned
    by the fact that the `read()` method for these objects block
    indefinitely if either no size is passed, or a size greater than
    the request's content length is passed to the method.

    This class normalizes *wsgi.input* behavior between WSGI servers
    by implementing non-blocking behavior for the cases mentioned
    above.

    Args:
        stream: Instance of ``socket._fileobject`` from
            ``environ['wsgi.input']``
        stream_len: Expected content length of the stream.

    """

    def __init__(self, stream, stream_len):
        self.stream = stream
        self.stream_len = stream_len

        self._bytes_remaining = self.stream_len

    def __iter__(self):
        return self

    def __next__(self):
        return next(self.stream)

    next = __next__

    def _read(self, size, target):
        """Helper function for proxing reads to the underlying stream.

        Args:
            size (int): Maximum number of bytes to read. Will be
                coerced, if None or -1, to the number of remaining bytes
                in the stream. Will likewise be coerced if greater than
                the number of remaining bytes, to avoid making a
                blocking call to the wrapped stream.
            target (callable): Once `size` has been fixed up, this function
                will be called to actually do the work.

        Returns:
            bytes: Data read from the stream, as returned by `target`.

        """

        # NOTE(kgriffs): Default to reading all remaining bytes if the
        # size is not specified or is out of bounds. This behaves
        # similarly to the IO streams passed in by non-wsgiref servers.
        if (size is None or size == -1 or size > self._bytes_remaining):
            size = self._bytes_remaining

        self._bytes_remaining -= size
        return target(size)

    def readable(self):
        """Always returns ``True``."""
        return True

    def seekable(self):
        """Always returns ``False``."""
        return False

    def writeable(self):
        """Always returns ``False``."""
        return False

    def read(self, size=None):
        """Read from the stream.

        Args:
            size (int): Maximum number of bytes/characters to read.
                Defaults to reading until EOF.

        Returns:
            bytes: Data read from the stream.

        """

        return self._read(size, self.stream.read)

    def readline(self, limit=None):
        """Read a line from the stream.

        Args:
            limit (int): Maximum number of bytes/characters to read.
                Defaults to reading until EOF.

        Returns:
            bytes: Data read from the stream.

        """

        return self._read(limit, self.stream.readline)

    def readlines(self, hint=None):
        """Read lines from the stream.

        Args:
            hint (int): Maximum number of bytes/characters to read.
                Defaults to reading until EOF.

        Returns:
            bytes: Data read from the stream.

        """

        return self._read(hint, self.stream.readlines)

    def write(self, data):
        """Always raises IOError; writing is not supported."""

        raise IOError('Stream is not writeable')


# NOTE(kgriffs): Alias for backwards-compat
Body = BoundedStream



# Copyright 2016 by Rackspace Hosting, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Copyright 2016 by Rackspace Hosting, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""WSGI test client utilities.

This package includes utilities for simulating HTTP requests against a
WSGI callable, without having to stand up a WSGI server.
"""

import json
import re
import sys
import wsgiref.validate

from six.moves import http_cookies

from falcon.testing import helpers
from falcon.testing.srmock import StartResponseMock
from falcon.util import CaseInsensitiveDict, http_date_to_dt, to_query_str


class Result(object):
    """Encapsulates the result of a simulated WSGI request.

    Args:
        iterable (iterable): An iterable that yields zero or more
            bytestrings, per PEP-3333
        status (str): An HTTP status string, including status code and
            reason string
        headers (list): A list of (header_name, header_value) tuples,
            per PEP-3333

    Attributes:
        status (str): HTTP status string given in the response
        status_code (int): The code portion of the HTTP status string
        headers (CaseInsensitiveDict): A case-insensitive dictionary
            containing all the headers in the response, except for
            cookies, which may be accessed via the `cookies`
            attribute.

            Note:

                Multiple instances of a header in the response are
                currently not supported; it is unspecified which value
                will "win" and be represented in `headers`.

        cookies (dict): A dictionary of
            :py:class:`falcon.testing.Cookie` values parsed from the
            response, by name.
        encoding (str): Text encoding of the response body, or ``None``
            if the encoding can not be determined.
        content (bytes): Raw response body, or ``bytes`` if the
            response body was empty.
        text (str): Decoded response body of type ``unicode``
            under Python 2.6 and 2.7, and of type ``str`` otherwise.
            If the content type does not specify an encoding, UTF-8 is
            assumed.
        json (dict): Deserialized JSON body. Raises an error if the
            response is not JSON.
    """

    def __init__(self, iterable, status, headers):
        self._text = None

        self._content = b''.join(iterable)
        if hasattr(iterable, 'close'):
            iterable.close()

        self._status = status
        self._status_code = int(status[:3])
        self._headers = CaseInsensitiveDict(headers)

        cookies = http_cookies.SimpleCookie()
        for name, value in headers:
            if name.lower() == 'set-cookie':
                cookies.load(value)

                if sys.version_info < (2, 7):
                    match = re.match('([^=]+)=', value)
                    assert match

                    cookie_name = match.group(1)

                    # NOTE(kgriffs): py26 has a bug that causes
                    # SimpleCookie to incorrectly parse the "expires"
                    # attribute, so we have to do it ourselves. This
                    # algorithm is obviously very naive, but it should
                    # work well enough until we stop supporting
                    # 2.6, at which time we can remove this code.
                    match = re.search('expires=([^;]+)', value)
                    if match:
                        cookies[cookie_name]['expires'] = match.group(1)

                    # NOTE(kgriffs): py26's SimpleCookie won't parse
                    # the "httponly" and "secure" attributes, so we
                    # have to do it ourselves.
                    if 'httponly' in value:
                        cookies[cookie_name]['httponly'] = True

                    if 'secure' in value:
                        cookies[cookie_name]['secure'] = True

        self._cookies = dict(
            (morsel.key, Cookie(morsel))
            for morsel in cookies.values()
        )

        self._encoding = helpers.get_encoding_from_headers(self._headers)

    @property
    def status(self):
        return self._status

    @property
    def status_code(self):
        return self._status_code

    @property
    def headers(self):
        return self._headers

    @property
    def cookies(self):
        return self._cookies

    @property
    def encoding(self):
        return self._encoding

    @property
    def content(self):
        return self._content

    @property
    def text(self):
        if self._text is None:
            if not self.content:
                self._text = u''
            else:
                if self.encoding is None:
                    encoding = 'UTF-8'
                else:
                    encoding = self.encoding

                self._text = self.content.decode(encoding)

        return self._text

    @property
    def json(self):
        return json.loads(self.text)


class Cookie(object):
    """Represents a cookie returned by a simulated request.

    Args:
        morsel: A ``Morsel`` object from which to derive the cookie
            data.

    Attributes:
        name (str): The cookie's name.
        value (str): The value of the cookie.
        expires(datetime.datetime): Expiration timestamp for the cookie,
            or ``None`` if not specified.
        path (str): The path prefix to which this cookie is restricted,
            or ``None`` if not specified.
        domain (str): The domain to which this cookie is restricted,
            or ``None`` if not specified.
        max_age (int): The lifetime of the cookie in seconds, or
            ``None`` if not specified.
        secure (bool): Whether or not the cookie may only only be
            transmitted from the client via HTTPS.
        http_only (bool): Whether or not the cookie may only be
            included in unscripted requests from the client.
    """

    def __init__(self, morsel):
        self._name = morsel.key
        self._value = morsel.value

        for name in (
            'expires',
            'path',
            'domain',
            'max_age',
            'secure',
            'httponly',
        ):
            value = morsel[name.replace('_', '-')] or None
            setattr(self, '_' + name, value)

    @property
    def name(self):
        return self._name

    @property
    def value(self):
        return self._value

    @property
    def expires(self):
        if self._expires:
            return http_date_to_dt(self._expires, obs_date=True)

        return None

    @property
    def path(self):
        return self._path

    @property
    def domain(self):
        return self._domain

    @property
    def max_age(self):
        return int(self._max_age) if self._max_age else None

    @property
    def secure(self):
        return bool(self._secure)

    @property
    def http_only(self):
        return bool(self._httponly)


def simulate_request(app, method='GET', path='/', query_string=None,
                     headers=None, body=None, file_wrapper=None,
                     params=None, params_csv=True):
        """Simulates a request to a WSGI application.

        Performs a request against a WSGI application. Uses
        :any:`wsgiref.validate` to ensure the response is valid
        WSGI.

        Keyword Args:
            app (callable): The WSGI application to call
            method (str): An HTTP method to use in the request
                (default: 'GET')
            path (str): The URL path to request (default: '/')
            params (dict): A dictionary of query string parameters,
                where each key is a parameter name, and each value is
                either a ``str`` or something that can be converted
                into a ``str``, or a list of such values. If a ``list``,
                the value will be converted to a comma-delimited string
                of values (e.g., 'thing=1,2,3').
            params_csv (bool): Set to ``False`` to encode list values
                in query string params by specifying multiple instances
                of the parameter (e.g., 'thing=1&thing=2&thing=3').
                Otherwise, parameters will be encoded as comma-separated
                values (e.g., 'thing=1,2,3'). Defaults to ``True``.
            query_string (str): A raw query string to include in the
                request (default: ``None``). If specified, overrides
                `params`.
            headers (dict): Additional headers to include in the request
                (default: ``None``)
            body (str): A string to send as the body of the request.
                Accepts both byte strings and Unicode strings
                (default: ``None``). If a Unicode string is provided,
                it will be encoded as UTF-8 in the request.
            file_wrapper (callable): Callable that returns an iterable,
                to be used as the value for *wsgi.file_wrapper* in the
                environ (default: ``None``). This can be used to test
                high-performance file transmission when `resp.stream` is
                set to a file-like object.

        Returns:
            :py:class:`~.Result`: The result of the request
        """

        if not path.startswith('/'):
            raise ValueError("path must start with '/'")

        if query_string and query_string.startswith('?'):
            raise ValueError("query_string should not start with '?'")

        if '?' in path:
            # NOTE(kgriffs): We could allow this, but then we'd need
            #   to define semantics regarding whether the path takes
            #   precedence over the query_string. Also, it would make
            #   tests less consistent, since there would be "more than
            #   one...way to do it."
            raise ValueError(
                'path may not contain a query string. Please use the '
                'query_string parameter instead.'
            )

        if query_string is None:
            query_string = to_query_str(
                params,
                comma_delimited_lists=params_csv,
                prefix=False,
            )

        env = helpers.create_environ(
            method=method,
            path=path,
            query_string=(query_string or ''),
            headers=headers,
            body=body,
            file_wrapper=file_wrapper,
        )

        srmock = StartResponseMock()
        validator = wsgiref.validate.validator(app)
        iterable = validator(env, srmock)

        result = Result(iterable, srmock.status, srmock.headers)

        return result


def simulate_get(app, path, **kwargs):
    """Simulates a GET request to a WSGI application.

    Equivalent to::

         simulate_request(app, 'GET', path, **kwargs)

    Args:
        app (callable): The WSGI application to call
        path (str): The URL path to request

    Keyword Args:
        params (dict): A dictionary of query string parameters,
            where each key is a parameter name, and each value is
            either a ``str`` or something that can be converted
            into a ``str``, or a list of such values. If a ``list``,
            the value will be converted to a comma-delimited string
            of values (e.g., 'thing=1,2,3').
        params_csv (bool): Set to ``False`` to encode list values
            in query string params by specifying multiple instances
            of the parameter (e.g., 'thing=1&thing=2&thing=3').
            Otherwise, parameters will be encoded as comma-separated
            values (e.g., 'thing=1,2,3'). Defaults to ``True``.
        query_string (str): A raw query string to include in the
            request (default: ``None``). If specified, overrides
            `params`.
        headers (dict): Additional headers to include in the request
            (default: ``None``)
        file_wrapper (callable): Callable that returns an iterable,
            to be used as the value for *wsgi.file_wrapper* in the
            environ (default: ``None``). This can be used to test
            high-performance file transmission when `resp.stream` is
            set to a file-like object.
    """
    return simulate_request(app, 'GET', path, **kwargs)


def simulate_head(app, path, **kwargs):
    """Simulates a HEAD request to a WSGI application.

    Equivalent to::

         simulate_request(app, 'HEAD', path, **kwargs)

    Args:
        app (callable): The WSGI application to call
        path (str): The URL path to request

    Keyword Args:
        params (dict): A dictionary of query string parameters,
            where each key is a parameter name, and each value is
            either a ``str`` or something that can be converted
            into a ``str``, or a list of such values. If a ``list``,
            the value will be converted to a comma-delimited string
            of values (e.g., 'thing=1,2,3').
        params_csv (bool): Set to ``False`` to encode list values
            in query string params by specifying multiple instances
            of the parameter (e.g., 'thing=1&thing=2&thing=3').
            Otherwise, parameters will be encoded as comma-separated
            values (e.g., 'thing=1,2,3'). Defaults to ``True``.
        query_string (str): A raw query string to include in the
            request (default: ``None``). If specified, overrides
            `params`.
        headers (dict): Additional headers to include in the request
            (default: ``None``)
    """
    return simulate_request(app, 'HEAD', path, **kwargs)


def simulate_post(app, path, **kwargs):
    """Simulates a POST request to a WSGI application.

    Equivalent to::

         simulate_request(app, 'POST', path, **kwargs)

    Args:
        app (callable): The WSGI application to call
        path (str): The URL path to request

    Keyword Args:
        params (dict): A dictionary of query string parameters,
            where each key is a parameter name, and each value is
            either a ``str`` or something that can be converted
            into a ``str``, or a list of such values. If a ``list``,
            the value will be converted to a comma-delimited string
            of values (e.g., 'thing=1,2,3').
        params_csv (bool): Set to ``False`` to encode list values
            in query string params by specifying multiple instances
            of the parameter (e.g., 'thing=1&thing=2&thing=3').
            Otherwise, parameters will be encoded as comma-separated
            values (e.g., 'thing=1,2,3'). Defaults to ``True``.
        headers (dict): Additional headers to include in the request
            (default: ``None``)
        body (str): A string to send as the body of the request.
            Accepts both byte strings and Unicode strings
            (default: ``None``). If a Unicode string is provided,
            it will be encoded as UTF-8 in the request.
    """
    return simulate_request(app, 'POST', path, **kwargs)


def simulate_put(app, path, **kwargs):
    """Simulates a PUT request to a WSGI application.

    Equivalent to::

         simulate_request(app, 'PUT', path, **kwargs)

    Args:
        app (callable): The WSGI application to call
        path (str): The URL path to request

    Keyword Args:
        params (dict): A dictionary of query string parameters,
            where each key is a parameter name, and each value is
            either a ``str`` or something that can be converted
            into a ``str``, or a list of such values. If a ``list``,
            the value will be converted to a comma-delimited string
            of values (e.g., 'thing=1,2,3').
        params_csv (bool): Set to ``False`` to encode list values
            in query string params by specifying multiple instances
            of the parameter (e.g., 'thing=1&thing=2&thing=3').
            Otherwise, parameters will be encoded as comma-separated
            values (e.g., 'thing=1,2,3'). Defaults to ``True``.
        headers (dict): Additional headers to include in the request
            (default: ``None``)
        body (str): A string to send as the body of the request.
            Accepts both byte strings and Unicode strings
            (default: ``None``). If a Unicode string is provided,
            it will be encoded as UTF-8 in the request.
    """
    return simulate_request(app, 'PUT', path, **kwargs)


def simulate_options(app, path, **kwargs):
    """Simulates an OPTIONS request to a WSGI application.

    Equivalent to::

         simulate_request(app, 'OPTIONS', path, **kwargs)

    Args:
        app (callable): The WSGI application to call
        path (str): The URL path to request

    Keyword Args:
        params (dict): A dictionary of query string parameters,
            where each key is a parameter name, and each value is
            either a ``str`` or something that can be converted
            into a ``str``, or a list of such values. If a ``list``,
            the value will be converted to a comma-delimited string
            of values (e.g., 'thing=1,2,3').
        params_csv (bool): Set to ``False`` to encode list values
            in query string params by specifying multiple instances
            of the parameter (e.g., 'thing=1&thing=2&thing=3').
            Otherwise, parameters will be encoded as comma-separated
            values (e.g., 'thing=1,2,3'). Defaults to ``True``.
        headers (dict): Additional headers to include in the request
            (default: ``None``)
    """
    return simulate_request(app, 'OPTIONS', path, **kwargs)


def simulate_patch(app, path, **kwargs):
    """Simulates a PATCH request to a WSGI application.

    Equivalent to::

         simulate_request(app, 'PATCH', path, **kwargs)

    Args:
        app (callable): The WSGI application to call
        path (str): The URL path to request

    Keyword Args:
        params (dict): A dictionary of query string parameters,
            where each key is a parameter name, and each value is
            either a ``str`` or something that can be converted
            into a ``str``, or a list of such values. If a ``list``,
            the value will be converted to a comma-delimited string
            of values (e.g., 'thing=1,2,3').
        params_csv (bool): Set to ``False`` to encode list values
            in query string params by specifying multiple instances
            of the parameter (e.g., 'thing=1&thing=2&thing=3').
            Otherwise, parameters will be encoded as comma-separated
            values (e.g., 'thing=1,2,3'). Defaults to ``True``.
        headers (dict): Additional headers to include in the request
            (default: ``None``)
        body (str): A string to send as the body of the request.
            Accepts both byte strings and Unicode strings
            (default: ``None``). If a Unicode string is provided,
            it will be encoded as UTF-8 in the request.
    """
    return simulate_request(app, 'PATCH', path, **kwargs)


def simulate_delete(app, path, **kwargs):
    """Simulates a DELETE request to a WSGI application.

    Equivalent to::

         simulate_request(app, 'DELETE', path, **kwargs)

    Args:
        app (callable): The WSGI application to call
        path (str): The URL path to request

    Keyword Args:
        params (dict): A dictionary of query string parameters,
            where each key is a parameter name, and each value is
            either a ``str`` or something that can be converted
            into a ``str``, or a list of such values. If a ``list``,
            the value will be converted to a comma-delimited string
            of values (e.g., 'thing=1,2,3').
        params_csv (bool): Set to ``False`` to encode list values
            in query string params by specifying multiple instances
            of the parameter (e.g., 'thing=1&thing=2&thing=3').
            Otherwise, parameters will be encoded as comma-separated
            values (e.g., 'thing=1,2,3'). Defaults to ``True``.
        headers (dict): Additional headers to include in the request
            (default: ``None``)
    """
    return simulate_request(app, 'DELETE', path, **kwargs)


class TestClient(object):
    """Simulates requests to a WSGI application.

    This class provides a contextual wrapper for Falcon's `simulate_*`
    test functions. It lets you replace this::

        simulate_get(app, '/messages')
        simulate_head(app, '/messages')

    with this::

        client = TestClient(app)
        client.simulate_get('/messages')
        client.simulate_head('/messages')

    Args:
        app (callable): A WSGI application to target when simulating
            requests
    """

    def __init__(self, app):
        self.app = app

    def simulate_get(self, path='/', **kwargs):
        """Simulates a GET request to a WSGI application.

        See also: :py:meth:`falcon.testing.simulate_get`.
        """
        return simulate_get(self.app, path, **kwargs)

    def simulate_head(self, path='/', **kwargs):
        """Simulates a HEAD request to a WSGI application.

        See also: :py:meth:`falcon.testing.simulate_head`.
        """
        return simulate_head(self.app, path, **kwargs)

    def simulate_post(self, path='/', **kwargs):
        """Simulates a POST request to a WSGI application.

        See also: :py:meth:`falcon.testing.simulate_post`.
        """
        return simulate_post(self.app, path, **kwargs)

    def simulate_put(self, path='/', **kwargs):
        """Simulates a PUT request to a WSGI application.

        See also: :py:meth:`falcon.testing.simulate_put`.
        """
        return simulate_put(self.app, path, **kwargs)

    def simulate_options(self, path='/', **kwargs):
        """Simulates an OPTIONS request to a WSGI application.

        See also: :py:meth:`falcon.testing.simulate_options`.
        """
        return simulate_options(self.app, path, **kwargs)

    def simulate_patch(self, path='/', **kwargs):
        """Simulates a PATCH request to a WSGI application.

        See also: :py:meth:`falcon.testing.simulate_patch`.
        """
        return simulate_patch(self.app, path, **kwargs)

    def simulate_delete(self, path='/', **kwargs):
        """Simulates a DELETE request to a WSGI application.

        See also: :py:meth:`falcon.testing.simulate_delete`.
        """
        return simulate_delete(self.app, path, **kwargs)

    def simulate_request(self, *args, **kwargs):
        """Simulates a request to a WSGI application.

        Wraps :py:meth:`falcon.testing.simulate_request` to perform a
        WSGI request directly against ``self.app``. Equivalent to::

            falcon.testing.simulate_request(self.app, *args, **kwargs)
        """

        return simulate_request(self.app, *args, **kwargs)



from functools import wraps

from testtools.matchers import Contains

import falcon
import falcon.testing as testing

HTTP_METHODS = (
    'CONNECT',
    'DELETE',
    'GET',
    'HEAD',
    'OPTIONS',
    'POST',
    'PUT',
    'TRACE',
    'PATCH'
)


class ThingsResource(object):
    def __init__(self):
        self.called = False

        # Test non-callable attribute
        self.on_patch = {}

    # Field names ordered differently than in uri template
    def on_get(self, req, resp, sid, id):
        self.called = True

        self.req, self.resp = req, resp
        resp.status = falcon.HTTP_204

    # Field names ordered the same as in uri template
    def on_head(self, req, resp, id, sid):
        self.called = True

        self.req, self.resp = req, resp
        resp.status = falcon.HTTP_204

    def on_put(self, req, resp, id, sid):
        self.called = True

        self.req, self.resp = req, resp
        resp.status = falcon.HTTP_201


class Stonewall(object):
    pass


def capture(func):
    @wraps(func)
    def with_capture(*args, **kwargs):
        self = args[0]
        self.called = True
        self.req, self.resp = args[1:]
        func(*args, **kwargs)

    return with_capture


def selfless_decorator(func):
    def faulty(req, resp, foo, bar):
        pass

    return faulty


class MiscResource(object):
    def __init__(self):
        self.called = False

    @capture
    def on_get(self, req, resp):
        resp.status = falcon.HTTP_204

    @capture
    def on_head(self, req, resp):
        resp.status = falcon.HTTP_204

    @capture
    def on_put(self, req, resp):
        resp.status = falcon.HTTP_400

    @capture
    def on_patch(self, req, resp):
        pass

    def on_options(self, req, resp):
        resp.status = falcon.HTTP_204

        # NOTE(kgriffs): This is incorrect, but only return GET so
        # that we can verify that the default OPTIONS responder has
        # been overridden.
        resp.set_header('allow', 'GET')


class GetWithFaultyPutResource(object):
    def __init__(self):
        self.called = False

    @capture
    def on_get(self, req, resp):
        resp.status = falcon.HTTP_204

    def on_put(self, req, resp, param):
        raise TypeError()


class FaultyDecoratedResource(object):

    @selfless_decorator
    def on_get(self, req, resp):
        pass


class TestHttpMethodRouting(testing.TestBase):

    def before(self):
        self.api.add_route('/stonewall', Stonewall())

        self.resource_things = ThingsResource()
        self.api.add_route('/things', self.resource_things)
        self.api.add_route('/things/{id}/stuff/{sid}', self.resource_things)

        self.resource_misc = MiscResource()
        self.api.add_route('/misc', self.resource_misc)

        self.resource_get_with_faulty_put = GetWithFaultyPutResource()
        self.api.add_route('/get_with_param/{param}',
                           self.resource_get_with_faulty_put)

    def test_get(self):
        self.simulate_request('/things/42/stuff/57')
        self.assertEqual(self.srmock.status, falcon.HTTP_204)
        self.assertTrue(self.resource_things.called)

    def test_put(self):
        self.simulate_request('/things/42/stuff/1337', method='PUT')
        self.assertEqual(self.srmock.status, falcon.HTTP_201)
        self.assertTrue(self.resource_things.called)

    def test_post_not_allowed(self):
        self.simulate_request('/things/42/stuff/1337', method='POST')
        self.assertEqual(self.srmock.status, falcon.HTTP_405)
        self.assertFalse(self.resource_things.called)

    def test_misc(self):
        for method in ['GET', 'HEAD', 'PUT', 'PATCH']:
            self.resource_misc.called = False
            self.simulate_request('/misc', method=method)
            self.assertTrue(self.resource_misc.called)
            self.assertEqual(self.resource_misc.req.method, method)

    def test_methods_not_allowed_simple(self):
        for method in ['GET', 'HEAD', 'PUT', 'PATCH']:
            self.simulate_request('/stonewall', method=method)
            self.assertEqual(self.srmock.status, falcon.HTTP_405)

    def test_methods_not_allowed_complex(self):
        for method in HTTP_METHODS:
            if method in ('GET', 'PUT', 'HEAD', 'OPTIONS'):
                continue

            self.resource_things.called = False
            self.simulate_request('/things/84/stuff/65', method=method)

            self.assertFalse(self.resource_things.called)
            self.assertEqual(self.srmock.status, falcon.HTTP_405)

            headers = self.srmock.headers
            allow_header = ('allow', 'GET, HEAD, PUT, OPTIONS')

            self.assertThat(headers, Contains(allow_header))

    def test_method_not_allowed_with_param(self):
        for method in HTTP_METHODS:
            if method in ('GET', 'PUT', 'OPTIONS'):
                continue

            self.resource_get_with_faulty_put.called = False
            self.simulate_request(
                '/get_with_param/bogus_param', method=method)

            self.assertFalse(self.resource_get_with_faulty_put.called)
            self.assertEqual(self.srmock.status, falcon.HTTP_405)

            headers = self.srmock.headers
            allow_header = ('allow', 'GET, PUT, OPTIONS')

            self.assertThat(headers, Contains(allow_header))

    def test_on_options(self):
        self.simulate_request('/things/84/stuff/65', method='OPTIONS')
        self.assertEqual(self.srmock.status, falcon.HTTP_204)

        headers = self.srmock.headers
        allow_header = ('allow', 'GET, HEAD, PUT')

        self.assertThat(headers, Contains(allow_header))

    def test_default_on_options(self):
        self.simulate_request('/misc', method='OPTIONS')
        self.assertEqual(self.srmock.status, falcon.HTTP_204)

        headers = self.srmock.headers
        allow_header = ('allow', 'GET')

        self.assertThat(headers, Contains(allow_header))

    def test_bogus_method(self):
        self.simulate_request('/things', method=self.getUniqueString())
        self.assertFalse(self.resource_things.called)
        self.assertEqual(self.srmock.status, falcon.HTTP_400)



import falcon
from falcon import testing


class TestCustomRouter(testing.TestBase):

    def test_custom_router_add_route_should_be_used(self):

        check = []

        class CustomRouter(object):
            def add_route(self, uri_template, *args, **kwargs):
                check.append(uri_template)

        api = falcon.API(router=CustomRouter())
        api.add_route('/test', 'resource')

        self.assertEqual(len(check), 1)
        self.assertIn('/test', check)

    def test_custom_router_find_should_be_used(self):

        def resource(req, resp, **kwargs):
            resp.body = '{{"uri_template": "{0}"}}'.format(req.uri_template)

        class CustomRouter(object):
            def __init__(self):
                self.reached_backwards_compat = False

            def find(self, uri):
                if uri == '/test/42':
                    return resource, {'GET': resource}, {}, '/test/{id}'

                if uri == '/test/42/no-uri-template':
                    return resource, {'GET': resource}, {}, None

                if uri == '/test/42/uri-template/backwards-compat':
                    return resource, {'GET': resource}, {}

                if uri == '/404/backwards-compat':
                    self.reached_backwards_compat = True
                    return (None, None, None)

                return None

        router = CustomRouter()
        self.api = falcon.API(router=router)
        body = self.simulate_request('/test/42')
        self.assertEqual(body, [b'{"uri_template": "/test/{id}"}'])

        body = self.simulate_request('/test/42/no-uri-template')
        self.assertEqual(body, [b'{"uri_template": "None"}'])

        body = self.simulate_request('/test/42/uri-template/backwards-compat')
        self.assertEqual(body, [b'{"uri_template": "None"}'])

        for uri in ('/404', '/404/backwards-compat'):
            body = self.simulate_request(uri)
            self.assertFalse(body)
            self.assertEqual(self.srmock.status, falcon.HTTP_404)

        self.assertTrue(router.reached_backwards_compat)

    def test_can_pass_additional_params_to_add_route(self):

        check = []

        class CustomRouter(object):
            def add_route(self, uri_template, method_map, resource, name):
                self._index = {name: uri_template}
                check.append(name)

        api = falcon.API(router=CustomRouter())
        api.add_route('/test', 'resource', name='my-url-name')

        self.assertEqual(len(check), 1)
        self.assertIn('my-url-name', check)

        # Also as arg.
        api.add_route('/test', 'resource', 'my-url-name-arg')

        self.assertEqual(len(check), 2)
        self.assertIn('my-url-name-arg', check)



# Copyright 2013 by Rackspace Hosting, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Utilities for the API class."""

from functools import wraps

from falcon import util


def prepare_middleware(middleware=None):
    """Check middleware interface and prepare it to iterate.

    Args:
        middleware:  list (or object) of input middleware

    Returns:
        list: A list of prepared middleware tuples
    """

    # PERF(kgriffs): do getattr calls once, in advance, so we don't
    # have to do them every time in the request path.
    prepared_middleware = []

    if middleware is None:
        middleware = []
    else:
        if not isinstance(middleware, list):
            middleware = [middleware]

    for component in middleware:
        process_request = util.get_bound_method(component,
                                                'process_request')
        process_resource = util.get_bound_method(component,
                                                 'process_resource')
        process_response = util.get_bound_method(component,
                                                 'process_response')

        if not (process_request or process_resource or process_response):
            msg = '{0} does not implement the middleware interface'
            raise TypeError(msg.format(component))

        if process_response:
            # NOTE(kgriffs): Shim older implementations to ensure
            # backwards-compatibility.
            args = util.get_argnames(process_response)

            if len(args) == 3:  # (req, resp, resource)
                def let(process_response=process_response):
                    @wraps(process_response)
                    def shim(req, resp, resource, req_succeeded):
                        process_response(req, resp, resource)

                    return shim

                process_response = let()

        prepared_middleware.append((process_request, process_resource,
                                    process_response))

    return prepared_middleware


def default_serialize_error(req, resp, exception):
    """Serialize the given instance of HTTPError.

    This function determines which of the supported media types, if
    any, are acceptable by the client, and serializes the error
    to the preferred type.

    Currently, JSON and XML are the only supported media types. If the
    client accepts both JSON and XML with equal weight, JSON will be
    chosen.

    Other media types can be supported by using a custom error serializer.

    Note:
        If a custom media type is used and the type includes a
        "+json" or "+xml" suffix, the error will be serialized
        to JSON or XML, respectively. If this behavior is not
        desirable, a custom error serializer may be used to
        override this one.

    Args:
        req: Instance of ``falcon.Request``
        resp: Instance of ``falcon.Response``
        exception: Instance of ``falcon.HTTPError``
    """
    representation = None

    preferred = req.client_prefers(('application/xml',
                                    'text/xml',
                                    'application/json'))

    if preferred is None:
        # NOTE(kgriffs): See if the client expects a custom media
        # type based on something Falcon supports. Returning something
        # is probably better than nothing, but if that is not
        # desired, this behavior can be customized by adding a
        # custom HTTPError serializer for the custom type.
        accept = req.accept.lower()

        # NOTE(kgriffs): Simple heuristic, but it's fast, and
        # should be sufficiently accurate for our purposes. Does
        # not take into account weights if both types are
        # acceptable (simply chooses JSON). If it turns out we
        # need to be more sophisticated, we can always change it
        # later (YAGNI).
        if '+json' in accept:
            preferred = 'application/json'
        elif '+xml' in accept:
            preferred = 'application/xml'

    if preferred is not None:
        resp.append_header('Vary', 'Accept')
        if preferred == 'application/json':
            representation = exception.to_json()
        else:
            representation = exception.to_xml()

        resp.body = representation
        resp.content_type = preferred + '; charset=UTF-8'


def wrap_old_error_serializer(old_fn):
    """Wraps an old-style error serializer to add body/content_type setting.

    Args:
        old_fn: Old-style error serializer

    Returns:
        A function that does the same, but sets body/content_type as needed.

    """
    def new_fn(req, resp, exception):
        media_type, body = old_fn(req, exception)
        if body is not None:
            resp.body = body
            resp.content_type = media_type

    return new_fn



def application(environ, start_response):
    # wsgi_errors = environ['wsgi.errors']

    start_response('200 OK', [
        ('Content-Type', 'text/plain')])

    body = '\n{\n'
    for key, value in environ.items():
        # if isinstance(value, str):
        body += '    "{0}": "{1}",\n'.format(key, value)

    body += '}\n\n'

    return [body.encode('utf-8')]

app = application


if __name__ == '__main__':
    # import eventlet.wsgi
    # import eventlet
    # eventlet.wsgi.server(eventlet.listen(('localhost', 8000)), application)

    from wsgiref.simple_server import make_server
    server = make_server('localhost', 8000, application)
    server.serve_forever()



import testtools
import falcon
import falcon.status_codes as status


class TestError(testtools.TestCase):
    def test_http_bad_request_no_title_and_desc(self):
        try:
            raise falcon.HTTPBadRequest()
        except falcon.HTTPBadRequest as e:
            self.assertEqual(status.HTTP_400, e.title,
                             'The title should be ' + status.HTTP_400 + ', but it is: ' + e.title)
            self.assertEqual(None, e.description, 'The description should be None')

    def test_http_bad_request_with_title_and_desc(self):
        try:
            raise falcon.HTTPBadRequest(title='Test', description='Testdescription')
        except falcon.HTTPBadRequest as e:
            self.assertEqual('Test', e.title, 'Title should be "Test"')
            self.assertEqual('Testdescription', e.description,
                             'Description should be "Testdescription"')

    def test_http_unauthorized_no_title_and_desc_and_challenges(self):
        try:
            raise falcon.HTTPUnauthorized()
        except falcon.HTTPUnauthorized as e:
            self.assertEqual(status.HTTP_401, e.title,
                             'The title should be ' + status.HTTP_401 + ', but it is: ' + e.title)
            self.assertEqual(None, e.description, 'The description should be None')
            self.assertNotIn('WWW-Authenticate', e.headers,
                             'Challenges should not be found in headers')

    def test_http_unauthorized_with_title_and_desc_and_challenges(self):
        try:
            raise falcon.HTTPUnauthorized(title='Test', description='Testdescription',
                                          challenges=['Testch'])
        except falcon.HTTPUnauthorized as e:
            self.assertEqual('Test', e.title, 'Title should be "Test"')
            self.assertEqual('Testdescription', e.description,
                             'Description should be "Testdescription"')
            self.assertEqual('Testch', e.headers['WWW-Authenticate'], 'Challenges should be None')

    def test_http_forbidden_no_title_and_desc_and_challenges(self):
        try:
            raise falcon.HTTPForbidden()
        except falcon.HTTPForbidden as e:
            self.assertEqual(status.HTTP_403, e.title,
                             'The title should be ' + status.HTTP_403 + ', but it is: ' + e.title)
            self.assertEqual(None, e.description, 'The description should be None')

    def test_http_forbidden_with_title_and_desc_and_challenges(self):
        try:
            raise falcon.HTTPForbidden(title='Test', description='Testdescription')
        except falcon.HTTPForbidden as e:
            self.assertEqual('Test', e.title, 'Title should be "Test"')
            self.assertEqual('Testdescription', e.description,
                             'Description should be "Testdescription"')

    def test_http_not_acceptable_no_title_and_desc_and_challenges(self):
        try:
            raise falcon.HTTPNotAcceptable()
        except falcon.HTTPNotAcceptable as e:
            self.assertEqual(None, e.description, 'The description should be None')

    def test_http_not_acceptable_with_title_and_desc_and_challenges(self):
        try:
            raise falcon.HTTPNotAcceptable(description='Testdescription')
        except falcon.HTTPNotAcceptable as e:
            self.assertEqual('Testdescription', e.description,
                             'Description should be "Testdescription"')

    def test_http_conflict_no_title_and_desc_and_challenges(self):
        try:
            raise falcon.HTTPConflict()
        except falcon.HTTPConflict as e:
            self.assertEqual(status.HTTP_409, e.title,
                             'The title should be ' + status.HTTP_409 + ', but it is: ' + e.title)
            self.assertEqual(None, e.description, 'The description should be None')

    def test_http_conflict_with_title_and_desc_and_challenges(self):
        try:
            raise falcon.HTTPConflict(title='Test', description='Testdescription')
        except falcon.HTTPConflict as e:
            self.assertEqual('Test', e.title, 'Title should be "Test"')
            self.assertEqual('Testdescription', e.description,
                             'Description should be "Testdescription"')

    def test_http_length_required_no_title_and_desc_and_challenges(self):
        try:
            raise falcon.HTTPLengthRequired()
        except falcon.HTTPLengthRequired as e:
            self.assertEqual(status.HTTP_411, e.title,
                             'The title should be ' + status.HTTP_411 + ', but it is: ' + e.title)
            self.assertEqual(None, e.description, 'The description should be None')

    def test_http_length_required_with_title_and_desc_and_challenges(self):
        try:
            raise falcon.HTTPLengthRequired(title='Test', description='Testdescription')
        except falcon.HTTPLengthRequired as e:
            self.assertEqual('Test', e.title, 'Title should be "Test"')
            self.assertEqual('Testdescription', e.description,
                             'Description should be "Testdescription"')

    def test_http_precondition_failed_no_title_and_desc_and_challenges(self):
        try:
            raise falcon.HTTPPreconditionFailed()
        except falcon.HTTPPreconditionFailed as e:
            self.assertEqual(status.HTTP_412, e.title,
                             'The title should be ' + status.HTTP_412 + ', but it is: ' + e.title)
            self.assertEqual(None, e.description, 'The description should be None')

    def test_http_precondition_faild_with_title_and_desc_and_challenges(self):
        try:
            raise falcon.HTTPPreconditionFailed(title='Test', description='Testdescription')
        except falcon.HTTPPreconditionFailed as e:
            self.assertEqual('Test', e.title, 'Title should be "Test"')
            self.assertEqual('Testdescription', e.description,
                             'Description should be "Testdescription"')

    def test_http_request_entity_too_large_no_title_and_desc_and_challenges(self):
        try:
            raise falcon.HTTPRequestEntityTooLarge()
        except falcon.HTTPRequestEntityTooLarge as e:
            self.assertEqual(status.HTTP_413, e.title,
                             'The title should be ' + status.HTTP_413 + ', but it is: ' + e.title)
            self.assertEqual(None, e.description, 'The description should be None')
            self.assertNotIn('Retry-After', e.headers, 'Retry-After should not be in the headers')

    def test_http_request_entity_too_large_with_title_and_desc_and_challenges(self):
        try:
            raise falcon.HTTPRequestEntityTooLarge(title='Test', description='Testdescription',
                                                   retry_after=123)
        except falcon.HTTPRequestEntityTooLarge as e:
            self.assertEqual('Test', e.title, 'Title should be "Test"')
            self.assertEqual('Testdescription', e.description,
                             'Description should be "Testdescription"')
            self.assertEqual('123', e.headers['Retry-After'], 'Retry-After should be 123')

    def test_http_uri_too_long_no_title_and_desc_and_challenges(self):
        try:
            raise falcon.HTTPUriTooLong()
        except falcon.HTTPUriTooLong as e:
            self.assertEqual(status.HTTP_414, e.title,
                             'The title should be ' + status.HTTP_414 + ', but it is: ' + e.title)
            self.assertEqual(None, e.description, 'The description should be None')

    def test_http_uri_too_long_with_title_and_desc_and_challenges(self):
        try:
            raise falcon.HTTPUriTooLong(title='Test', description='Testdescription')
        except falcon.HTTPUriTooLong as e:
            self.assertEqual('Test', e.title, 'Title should be "Test"')
            self.assertEqual('Testdescription', e.description,
                             'Description should be "Testdescription"')

    def test_http_unsupported_media_type_no_title_and_desc_and_challenges(self):
        try:
            raise falcon.HTTPUnsupportedMediaType()
        except falcon.HTTPUnsupportedMediaType as e:
            self.assertEqual(None, e.description, 'The description should be None')

    def test_http_unsupported_media_type_with_title_and_desc_and_challenges(self):
        try:
            raise falcon.HTTPUnsupportedMediaType(description='Testdescription')
        except falcon.HTTPUnsupportedMediaType as e:
            self.assertEqual('Testdescription', e.description,
                             'Description should be "Testdescription"')

    def test_http_unprocessable_entity_no_title_and_desc_and_challenges(self):
        try:
            raise falcon.HTTPUnprocessableEntity()
        except falcon.HTTPUnprocessableEntity as e:
            self.assertEqual(status.HTTP_422, e.title,
                             'The title should be ' + status.HTTP_422 + ', but it is: ' + e.title)
            self.assertEqual(None, e.description, 'The description should be None')

    def test_http_unprocessable_with_title_and_desc_and_challenges(self):
        try:
            raise falcon.HTTPUnprocessableEntity(title='Test', description='Testdescription')
        except falcon.HTTPUnprocessableEntity as e:
            self.assertEqual('Test', e.title, 'Title should be "Test"')
            self.assertEqual('Testdescription', e.description,
                             'Description should be "Testdescription"')

    def test_http_too_many_requests_no_title_and_desc_and_challenges(self):
        try:
            raise falcon.HTTPTooManyRequests()
        except falcon.HTTPTooManyRequests as e:
            self.assertEqual(status.HTTP_429, e.title,
                             'The title should be ' + status.HTTP_429 + ', but it is: ' + e.title)
            self.assertEqual(None, e.description, 'The description should be None')
            self.assertNotIn('Retry-After', e.headers, 'Retry-After should not be in the headers')

    def test_http_too_many_requests_with_title_and_desc_and_challenges(self):
        try:
            raise falcon.HTTPTooManyRequests(title='Test', description='Testdescription',
                                             retry_after=123)
        except falcon.HTTPTooManyRequests as e:
            self.assertEqual('Test', e.title, 'Title should be "Test"')
            self.assertEqual('Testdescription', e.description,
                             'Description should be "Testdescription"')
            self.assertEqual('123', e.headers['Retry-After'], 'Retry-After should be 123')

    def test_http_unavailable_for_legal_reasons_no_title_and_desc_and_challenges(self):
        try:
            raise falcon.HTTPUnavailableForLegalReasons()
        except falcon.HTTPUnavailableForLegalReasons as e:
            self.assertEqual(status.HTTP_451, e.title,
                             'The title should be ' + status.HTTP_451 + ', but it is: ' + e.title)
            self.assertEqual(None, e.description, 'The description should be None')

    def test_http_unavailable_for_legal_reasons_with_title_and_desc_and_challenges(self):
        try:
            raise falcon.HTTPUnavailableForLegalReasons(title='Test',
                                                        description='Testdescription')
        except falcon.HTTPUnavailableForLegalReasons as e:
            self.assertEqual('Test', e.title, 'Title should be "Test"')
            self.assertEqual('Testdescription', e.description,
                             'Description should be "Testdescription"')

    def test_http_internal_server_error_entity_no_title_and_desc_and_challenges(self):
        try:
            raise falcon.HTTPInternalServerError()
        except falcon.HTTPInternalServerError as e:
            self.assertEqual(status.HTTP_500, e.title,
                             'The title should be ' + status.HTTP_500 + ', but it is: ' + e.title)
            self.assertEqual(None, e.description, 'The description should be None')

    def test_http_internal_server_error_with_title_and_desc_and_challenges(self):
        try:
            raise falcon.HTTPInternalServerError(title='Test', description='Testdescription')
        except falcon.HTTPInternalServerError as e:
            self.assertEqual('Test', e.title, 'Title should be "Test"')
            self.assertEqual('Testdescription', e.description,
                             'Description should be "Testdescription"')

    def test_http_bad_gateway_entity_no_title_and_desc_and_challenges(self):
        try:
            raise falcon.HTTPBadGateway()
        except falcon.HTTPBadGateway as e:
            self.assertEqual(status.HTTP_502, e.title,
                             'The title should be ' + status.HTTP_502 + ', but it is: ' + e.title)
            self.assertEqual(None, e.description, 'The description should be None')

    def test_http_bad_gateway_entity_with_title_and_desc_and_challenges(self):
        try:
            raise falcon.HTTPBadGateway(title='Test', description='Testdescription')
        except falcon.HTTPBadGateway as e:
            self.assertEqual('Test', e.title, 'Title should be "Test"')
            self.assertEqual('Testdescription', e.description,
                             'Description should be "Testdescription"')

    def test_http_service_unavailable_no_title_and_desc_and_challenges(self):
        try:
            raise falcon.HTTPServiceUnavailable()
        except falcon.HTTPServiceUnavailable as e:
            self.assertEqual(status.HTTP_503, e.title,
                             'The title should be ' + status.HTTP_503 + ', but it is: ' + e.title)
            self.assertEqual(None, e.description, 'The description should be None')
            self.assertNotIn('Retry-After', e.headers, 'Retry-After should not be in the headers')

    def test_http_service_unavailable_with_title_and_desc_and_challenges(self):
        try:
            raise falcon.HTTPServiceUnavailable(title='Test', description='Testdescription',
                                                retry_after=123)
        except falcon.HTTPServiceUnavailable as e:
            self.assertEqual('Test', e.title, 'Title should be "Test"')
            self.assertEqual('Testdescription', e.description,
                             'Description should be "Testdescription"')
            self.assertEqual('123', e.headers['Retry-After'], 'Retry-After should be 123')
